// (c) Copyright 2006.  Adobe Systems, Incorporated.  All rights reserved.

// written by Tom Ruark ( needs some major clean up )
// directed by Russell Preston Brown
// 08/28/2006 14:13

// debug level: 0-2 (0:disable, 1:break on error, 2:break at beginning)
// $.level = 0;
// debugger; // launch debugger on next line


//=================================================================
// Setup/Support
// This first portion of the script sets up an object to provide
// scope for all Dr. Brown BridgeTalk related routines to prevent
// name collision with other groups' scripts, defines some common
// utility functions, and adds some Dr. Brown related commands
// to the Bridge menus.
//=================================================================


try { // Overall try catch

	var drbrownserv = new Object;

	drbrownserv.version = "1.5";

	drbrownserv.menuItemInfoArray = new Array ();

	//-----------------------------------------------------------------
	// This routine sets the argument menu node to be enabled.
	//-----------------------------------------------------------------
	drbrownserv.alwaysEnabled = function () {
		this.enabled = true;
	}

	// some utility routines and properties

	// given a file name and a list of extensions
	// determine if this file is in the list of extensions
	drbrownserv.isFileOneOfThese = function ( inFileName, inArrayOfFileExtensions ) {
		var lastDot = inFileName.toString().lastIndexOf( "." );
		if ( lastDot == -1 ) {
			return false;
		}
		var strLength = inFileName.toString().length;
		var extension = inFileName.toString().substr( lastDot + 1, strLength - lastDot );
		extension = extension.toLowerCase();
		for (var i = 0; i < inArrayOfFileExtensions.length; i++ ) {
			if ( extension == inArrayOfFileExtensions[i] ) {
				return true;
			}
		}
		return false;
	}
	
	// fix it so we can save jpeg and cr files to jpeg without dialogs
	drbrownserv.saveFile = function( doc, jpegQ ) {
		if ( drbrownserv.isFileOneOfThese( doc.fullName, [ "jpg", "jpeg" ] ) ) {
			doc.saveAs( doc.fullName, jpegQ );
			doc.close(SaveOptions.DONOTSAVECHANGES);
		} else { 
			if ( drbrownserv.isFileOneOfThese( doc.fullName, drbrownserv.filesForCameraRaw ) ) {
				var fileToDelete = doc.fullName;
				
				// get the file without the path
				var strFileFullName = doc.fullName.toString();
				var lastDot = strFileFullName.lastIndexOf( "." );
				if ( lastDot == -1 ) {
					lastDot = strFileFullName.length;
				}
				var fullPathNoExtension = strFileFullName.substr( 0, lastDot );
				var newFile = File( fullPathNoExtension + ".jpg" );

				var newFileArray = drbrownserv.findNewHomeForFiles( [newFile], false, false, false, false); // files array, autoMode, makeFolder, bwMode, makeCopy

				doc.saveAs( newFileArray[ 0 ], jpegQ );
				doc.close(SaveOptions.DONOTSAVECHANGES);

				fileToDelete.remove(); 
			} else {
				doc.save();
				doc.close(SaveOptions.DONOTSAVECHANGES);
			}
		}
	}

	// ubber routine that should be broken up
	drbrownserv.findNewHomeForFiles = function( infiles, autoMode, makeFolder, bwMode, makeCopy ) {
		var outfiles = new Array();
		try {
			for (var i = 0; i < infiles.length; i++) {
				var destFolder = infiles[i].parent.toString();
				if (makeFolder) {
					if (bwMode) {
						destFolder += "/Black-n-White";
					} else {
						if (autoMode) {
							destFolder += "/Auto Corrected";
						} else {
							destFolder += "/Manual Corrected";
						}
					}
					if (!Folder(destFolder).exist) {
						Folder(destFolder).create();
					}
				}
				var fileName = infiles[i].fullName.toString();
				var lastDot = fileName.lastIndexOf( "." );
				if (lastDot == -1) {
					lastDot = fileName.length;
				}
				fileName = fileName.substr( 0, lastDot );
				var lastSlash = fileName.lastIndexOf( "/" );
				fileName = fileName.substr( lastSlash + 1, fileName.length );
				var extension = infiles[i].fullName.toString().substr( lastDot, infiles[i].fullName.toString().length);
				fileName = fileName.replace(/[:\/\\*\?\"\<\>\|]/g, "_");  // '/\:*?"<>|' -> '_'
				var uniqueFileName = destFolder + "/" + fileName + extension;
				var fileNumber = 1;
				while (File(uniqueFileName).exists) {
					uniqueFileName = destFolder + "/" + fileName + "_" + fileNumber + extension;
					fileNumber++;
				}
				outfiles.push(File(uniqueFileName));

				if (makeCopy) {
					infiles[i].copy(uniqueFileName);
				}

				//// don't mess with the metadata
				//var tn = new Thumbnail(File(uniqueFileName));
				//var m = tn.metadata;
				//m.namespace = "http://ns.adobe.com/xap/1.0/";
				//m.Label = "Red";
				//// end don't mess with the metadata
				// Window.alert('a');
			}
		}
		catch(error) {
			Window.alert(error + ":" + error.line);
		}
		return outfiles;
	}

	// A list of camera raw extensions, keep them lower case, plus jpeg
	drbrownserv.filesForCameraRaw = Array( "tif", "crw", "nef", "raf", "orf", "mrw", "dcr", "mos", "srf", "pef", "dcr", "cr2", "dng", "erf", "x3f", "raw" );


	//=================================================================
	// blackandwhiteFromBridge
	//=================================================================

	//-----------------------------------------------------------------
	//-----------------------------------------------------------------
	drbrownserv.blackandwhite = function (files) {
		try {
			if (BridgeTalk.appName != "photoshop") {
				// Bring Photoshop to the foreground.
				BridgeTalk.bringToFront ("photoshop");
		
				// Create a new BridgeTalk message for Photoshop to invoke
				// Image Processor with the selected files
				var btMessage = new BridgeTalk;
				btMessage.target = "photoshop";
				btMessage.body = "drbrownserv.blackandwhite (" + files.toSource () + ");";
				btMessage.send ();
			} else {
				alert('blackandwhitefrombridge');
			}
		}
		catch (error) {
			if (error.number != 8007) // Don't report user cancelled errors.
				alert (error);
		}
	}


	//-----------------------------------------------------------------
	// This routine is called when the Bridge's Dr. Brown's black and white
	// menu item is chosen.
	//-----------------------------------------------------------------
	drbrownserv.blackandwhiteFromBridge = function () {
		try {
			var files = photoshop.getBridgeFileListForAutomateCommand (false);
			
			if (files.length != 0) {
				// drbrownserv.blackandwhite (files);
				drbrownserv.miniprocessor (files, false, true, true); // files, autoMode, makeFolder, bwMode
			}
		}
		catch (error) {
			if (error.number != 8007) // Don't report user cancelled errors.
				alert (error);
		}
	}


	//=================================================================
	// manualcorrectFromBridge
	//=================================================================

	//-----------------------------------------------------------------
	//-----------------------------------------------------------------
	drbrownserv.manualcorrect = function (files) {
		try {
			if (BridgeTalk.appName != "photoshop") {
				Window.alert('manualcorrect ERROR');
				// Bring Photoshop to the foreground.
				BridgeTalk.bringToFront ("photoshop");
		
				// Create a new BridgeTalk message for Photoshop to invoke
				// Image Processor with the selected files
				var btMessage = new BridgeTalk;
				btMessage.target = "photoshop";
				btMessage.body = "drbrownserv.manualcorrect (" + files.toSource () + ");";
				btMessage.send ();
			} else {
				alert('manualcorrectFromBridge ERROR');
			}
		}
		catch (error) {
			if (error.number != 8007) // Don't report user cancelled errors.
				alert (error);
		}
	}


	//-----------------------------------------------------------------
	// This routine is called when the Bridge's Dr. Brown's manual correct
	// menu item is chosen.
	//-----------------------------------------------------------------
	drbrownserv.manualcorrectFromBridge = function () {
		try {
			var files = photoshop.getBridgeFileListForAutomateCommand (false);
			
			if (files.length != 0)
				drbrownserv.miniprocessor (files, false, true, false); // files, autoMode, makeFolder, bwMode
		}
		catch (error) {
			if (error.number != 8007) // Don't report user cancelled errors.
				alert (error);
		}
	}


	//=================================================================
	// autocorrectFromBridge
	//=================================================================

	//-----------------------------------------------------------------
	//-----------------------------------------------------------------
	drbrownserv.autocorrect = function (files) {
		try {
			if (BridgeTalk.appName != "photoshop") {
				Window.alert('autocorrectFromBridge BRIDGE ERROR');
				// Bring Photoshop to the foreground.
				BridgeTalk.bringToFront ("photoshop");
		
				// Create a new BridgeTalk message for Photoshop to invoke
				// Image Processor with the selected files
				var btMessage = new BridgeTalk;
				btMessage.target = "photoshop";
				btMessage.body = "drbrownserv.autocorrect (" + files.toSource () + ");";
				btMessage.send ();
			} else {
				alert('autocorrectFromBridge ERROR');
			}
		}
		catch (error) {
			if (error.number != 8007) // Don't report user cancelled errors.
				alert (error);
		}
	}


	//-----------------------------------------------------------------
	// This routine is called when the Bridge's Dr. Brown's auto correct
	// menu item is chosen.
	//-----------------------------------------------------------------
	drbrownserv.autocorrectFromBridge = function () {
		try {
			var files = photoshop.getBridgeFileListForAutomateCommand (false);
			
			if (files.length != 0)
				drbrownserv.miniprocessor (files, true, true, false); // files, autoMode, makeFolder, bwMode
		}
		catch (error) {
			if (error.number != 8007) // Don't report user cancelled errors.
				alert (error);
		}
	}


	//=================================================================
	// CaptionMaker
	// Sets up Dr. Brown's Caption Maker JavaScript to be
	// accessed from the Bridge.
	//=================================================================

	//-----------------------------------------------------------------
	// This routine takes an array of files. If called by Photoshop,
	// it will invoke Dr. Brown's Caption Maker with the files. If called by
	// any other app, it will send a BridgeTalk message to Photoshop
	// to invoke this routine with the same arguments.
	//-----------------------------------------------------------------
	drbrownserv.captionmaker = function (files) {
		try {
			if (BridgeTalk.appName != "photoshop") {
				// Bring Photoshop to the foreground.
				BridgeTalk.bringToFront ("photoshop");
		
				// Create a new BridgeTalk message for Photoshop to invoke
				// Image Processor with the selected files
				var btMessage = new BridgeTalk;
				btMessage.target = "photoshop";
				btMessage.body = "drbrownserv.captionmaker (" + files.toSource () + ");";
				btMessage.send ();
			} else {
		
				// Dr. Brown's Caption Maker script will recognize this and use it
				var gFilesFromBridge = files;
		
				var strPresets = localize ("$$$/ApplicationPresetsFolder/Presets=Presets");
				var strScripts = localize ("$$$/PSBI/Automate/ImageProcessor/Photoshop/Scripts=Scripts");
				var strCaptionMaker = "Dr. Brown's Caption Maker.jsx";
				
				var cmFile = new File (app.path + "/" + strPresets + "/" + strScripts + "/" + strCaptionMaker);
				
				var rememberDialogModes = displayDialogs;
				displayDialogs = DialogModes.ALL;
						
				if (cmFile.exists) {
					if (cmFile.open ('r')) {
						var script = cmFile.read ();
						cmFile.close ();
						eval (script);
					} else {
						alert ( strCaptionMaker + " could not be opened." );
					}
				} else {
					alert ( cmFile.fsName + " could not be found." );
				}
		
				displayDialogs = rememberDialogModes;
			}
		}
		catch (error) {
			if (error.number != 8007) // Don't report user cancelled errors.
				alert (error);
		}
	}


	//-----------------------------------------------------------------
	// This routine is called when the Bridge's Dr. Brown's Caption Maker
	// menu item is chosen.
	//-----------------------------------------------------------------
	drbrownserv.captionmakerFromBridge = function () {
		try {
			var files = photoshop.getBridgeFileListForAutomateCommand (false);
			
			if (files.length != 0)
				drbrownserv.captionmaker (files);
		}
		catch (error) {
			if (error.number != 8007) // Don't report user cancelled errors.
				alert (error);
		}
	}


	//=================================================================
	// Process123
	// Sets up Dr. Brown's Process 1-2-3 JavaScript to be
	// accessed from the Bridge.
	//=================================================================

	//-----------------------------------------------------------------
	// This routine takes an array of files. If called by Photoshop,
	// it will invoke Dr. Brown's Process 1-2-3 with the files. If called by
	// any other app, it will send a BridgeTalk message to Photoshop
	// to invoke this routine with the same arguments.
	//-----------------------------------------------------------------
	drbrownserv.process123 = function (files) {
		try {
			if (BridgeTalk.appName != "photoshop") {
				// Bring Photoshop to the foreground.
				BridgeTalk.bringToFront ("photoshop");
		
				// Create a new BridgeTalk message for Photoshop to invoke
				// Image Processor with the selected files
				var btMessage = new BridgeTalk;
				btMessage.target = "photoshop";
				btMessage.body = "drbrownserv.process123 (" + files.toSource () + ");";
				btMessage.send ();
			} else {
		
				// Dr. Brown's Process 1-2-3 script will recognize this and use it
				var gFilesFromBridge = files;
		
				var strPresets = localize ("$$$/ApplicationPresetsFolder/Presets=Presets");
				var strScripts = localize ("$$$/PSBI/Automate/ImageProcessor/Photoshop/Scripts=Scripts");
				var strProcess123 = "Dr. Brown's 1-2-3 Process.jsx";
				
				var cmFile = new File (app.path + "/" + strPresets + "/" + strScripts + "/" + strProcess123);
				
				var rememberDialogModes = displayDialogs;
				displayDialogs = DialogModes.ALL;
						
				if (cmFile.exists) {
					if (cmFile.open ('r')) {
						var script = cmFile.read ();
						cmFile.close ();
						eval (script);
					} else {
						alert ( strProcess123 + " could not be opened." );
					}
				} else {
					alert ( cmFile.fsName + " could not be found." );
				}
		
				displayDialogs = rememberDialogModes;
			}
		}
		catch (error) {
			if (error.number != 8007) // Don't report user cancelled errors.
				alert (error);
		}
	}


	//-----------------------------------------------------------------
	// This routine is called when the Bridge's Dr. Brown's Caption Maker
	// menu item is chosen.
	//-----------------------------------------------------------------
	drbrownserv.process123FromBridge = function () {
		try {
			var files = photoshop.getBridgeFileListForAutomateCommand (false);
			
			if (files.length != 0)
				drbrownserv.process123 (files);
		}
		catch (error) {
			if (error.number != 8007) // Don't report user cancelled errors.
				alert (error);
		}
	}


	//-----------------------------------------------------------------
	// This routine takes an array of files. If called by Photoshop,
	// it will invoke the merge routine with the files. If called by
	// any other app, it will send a BridgeTalk message to Photoshop
	// to invoke this routine with the same arguments.
	//-----------------------------------------------------------------
	drbrownserv.drBrownsMergeAMatic = function (/* Array */ files) {
		try {
			if (BridgeTalk.appName != "photoshop") {
				// Bring Photoshop to the foreground.
				BridgeTalk.bringToFront ("photoshop");
			
				// Create a new BridgeTalk message for Photoshop to invoke
				// this command on the other side of the else with the selected files
				var btMessage = new BridgeTalk;
				btMessage.target = "photoshop";
				btMessage.body = "drbrownserv.drBrownsMergeAMatic (" + files.toSource () + ");";
				btMessage.send();

			} else {

				if ( files == undefined ) return;
				if ( files.length == undefined ) return;
				if ( files.length == 0 ) return;

				var	firstDoc = open( files[ 0 ] );
			
				var targetDoc = firstDoc.duplicate();
			
				firstDoc.close( SaveOptions.DONOTSAVECHANGES );

				for ( var i = 1; i < files.length; i++ ) {
					try {
						var newDoc = open( files[ i ] );

						newDoc.selection.selectAll();

						newDoc.selection.copy();

						var docSplit = files[ i ].toString().split( "/" );

						var fileName = docSplit[ docSplit.length - 1 ];

						activeDocument = targetDoc;

						targetDoc.paste();

						targetDoc.activeLayer.name = decodeURI( fileName );

						newDoc.close( SaveOptions.DONOTSAVECHANGES );

					}
					
					catch(e) {
						// alert( "ERROR drBrownsMergeAMatic: " + e );
					}
				}
			}
		}
		
		catch (error) {
			if (error.number != 8007) // Don't report user cancelled errors.
				alert (error); //  + ":" + error.line);
		}
	}


	//-----------------------------------------------------------------
	// This routine is called when the Dr. Browns merge a matic menu
	// item is chosen.
	//-----------------------------------------------------------------
	drbrownserv.drBrownsMergeAMaticFromBridge = function () {
		try {
			var files = photoshop.getBridgeFileListForAutomateCommand (false);
			
			if (files.length != 0)
				drbrownserv.drBrownsMergeAMatic (files);
		}
		
		catch (error) {
			if (error.number != 8007) // Don't report user cancelled errors.
				alert (error); //  + ":" + error.line);
		}
	}

	//-----------------------------------------------------------------
	// This routine takes an array of files. If called by Photoshop,
	// it will convert each file to placed smart object on a layer. 
	// If called by any other app, it will send a BridgeTalk message
	// to Photoshop to invoke this routine with the same arguments.
	// If only one file is selected it will place until the user
	// selects cancel or done in the RAW dialog
	//-----------------------------------------------------------------
	drbrownserv.drBrownsPlaceAMatic = function (/* Array */ files, /* new doc depth */ depth) {
		try {
			if (BridgeTalk.appName != "photoshop") {
				// Bring Photoshop to the foreground.
				BridgeTalk.bringToFront ("photoshop");
			
				// Create a new BridgeTalk message for Photoshop to invoke
				// this routine on the other side of the else
				// with the selected files
				var btMessage = new BridgeTalk;
				btMessage.target = "photoshop";
				btMessage.body = "drbrownserv.drBrownsPlaceAMatic (" + files.toSource () + ", " + depth + ");";
				btMessage.send();

			} else {

				if ( files == undefined ) return;
				if ( files.length == undefined ) return;
				if ( files.length == 0 ) return;

				var PlaceFile = function( inFile ) {
					var id18 = charIDToTypeID( "Plc " );
					var desc = new ActionDescriptor();
					var id19 = charIDToTypeID( "null" );
					desc.putPath( id19, inFile );
					var id20 = charIDToTypeID( "FTcs" );
					var id21 = charIDToTypeID( "QCSt" );
					var id22 = charIDToTypeID( "Qcsa" );
					desc.putEnumerated( id20, id21, id22 ); // not sure what this does
					var id23 = charIDToTypeID( "Ofst" );
					var descObj = new ActionDescriptor();
					var id24 = charIDToTypeID( "Hrzn" );
					var id25 = charIDToTypeID( "#Pxl" );
					descObj.putUnitDouble( id24, id25, 0.000000 ); // could add that to the call
					var id26 = charIDToTypeID( "Vrtc" );
					var id27 = charIDToTypeID( "#Pxl" );
					descObj.putUnitDouble( id26, id27, 0.000000 ); // and maybe this
					var id28 = charIDToTypeID( "Ofst" );
					desc.putObject( id23, id28, descObj );
					var descResult = executeAction( id18, desc, DialogModes.NO );
					desc = null;
					descObj = null;
					descResult = null;
					$.gc();
				}

				var firstDoc = open( files[ 0 ] );

				// switch the units, we can't handle percentage units
				var startRulerUnits = preferences.rulerUnits;
				preferences.rulerUnits = Units.INCHES;

				var w = firstDoc.width;
				var h = firstDoc.height;
				var r = firstDoc.resolution;

				preferences.rulerUnits = startRulerUnits;

				firstDoc.close( SaveOptions.DONOTSAVECHANGES );

				firstDoc = documents.add( w, h, r, "Dr. Brown's Place-A-Matic" );

				if (depth == 16) {
					// Convert the doc to 16-bit.
					app.activeDocument.bitsPerChannel = BitsPerChannelType.SIXTEEN;
				}

				firstLayer = app.activeDocument.activeLayer;


				if ( files.length == 1 ) {

					try {
						// ideally we would like to loop and have the user cancel
						// when they have placed enough of them
						// the problem is that non-camera raw files will not pop
						// a dialog and could get out of control
						// a work around would be to put a timer or counter and
						// try to stop when things get out of control
						// for now we will just place two of them
						// while (true) {
							PlaceFile( files[ 0 ] );
							PlaceFile( files[ 0 ] );
						// }
					}

					catch(error) {
						// alert (error + "A:A" + error.line);
					}

				} else {
					
					for ( var i = 0; i < files.length; i++ ) {
			
						try {
							PlaceFile( files[ i ] );
						}
						
						catch(error) {
							// alert (error + ":T:T:" + error.line);
						}
					}

				}

				firstLayer.remove();

				delete PlaceFile;

			}
		}
		
		catch (error) {
			if (error.number != 8007) // Don't report user cancelled errors.
				alert (error); //  + ":" + error.line);
		}
	}


	//-----------------------------------------------------------------
	// This routine is called when the Bridge's PDF Presentation menu
	// item is chosen.
	//-----------------------------------------------------------------
	drbrownserv.drBrownsPlaceAMaticFromBridge8 = function () {
		try {
			var files = photoshop.getBridgeFileListForAutomateCommand (false);

			if (files.length != 0)
				drbrownserv.drBrownsPlaceAMatic (files, 8);
		}
		
		catch (error) {
			if (error.number != 8007) // Don't report user cancelled errors.
				alert (error); //  + ":" + error.line);
		}

	}

	//-----------------------------------------------------------------
	// This routine is called when the Bridge's PDF Presentation menu
	// item is chosen.
	//-----------------------------------------------------------------
	drbrownserv.drBrownsPlaceAMaticFromBridge16 = function () {
		try {
			var files = photoshop.getBridgeFileListForAutomateCommand (false);

			if (files.length != 0)
				drbrownserv.drBrownsPlaceAMatic (files, 16);
		}
		
		catch (error) {
			if (error.number != 8007) // Don't report user cancelled errors.
				alert (error); //  + ":" + error.line);
		}

	}

	//-----------------------------------------------------------------
	// The code below inserts menus in reverse order
	//-----------------------------------------------------------------

	//-----------------------------------------------------------------
	// The code below inserts
	//-----------------------------------------------------------------
	if (BridgeTalk.appName == "bridge") {	
		// Use temp function to keep vars out of global namespace.
		drbrownserv.tempFunction = function () {
			// Set up the info necessary for inserting this item into the Bridge's menus later.
			var menuItemInfo = new Object;
			menuItemInfo.text = "Dr. Brown's Black-n-White";
			menuItemInfo.name = 'DrBrownBlackAndWhite';
			menuItemInfo.onDisplay = drbrownserv.alwaysEnabled;
			menuItemInfo.onSelect = drbrownserv.blackandwhiteFromBridge;
			
			drbrownserv.menuItemInfoArray.push (menuItemInfo);
			}
		
		drbrownserv.tempFunction ();
		delete drbrownserv.tempFunction;
		}


	//-----------------------------------------------------------------
	// The code below inserts
	//-----------------------------------------------------------------
	if (BridgeTalk.appName == "bridge") {	
		// Use temp function to keep vars out of global namespace.
		drbrownserv.tempFunction = function () {
			// Set up the info necessary for inserting this item into the Bridge's menus later.
			var menuItemInfo = new Object;
			menuItemInfo.text = "Dr. Brown's Manual Correct";
			menuItemInfo.name = 'DrBrownManualCorrect';
			menuItemInfo.onDisplay = drbrownserv.alwaysEnabled;
			menuItemInfo.onSelect = drbrownserv.manualcorrectFromBridge;
			
			drbrownserv.menuItemInfoArray.push (menuItemInfo);
			}
		
		drbrownserv.tempFunction ();
		delete drbrownserv.tempFunction;
		}


	//-----------------------------------------------------------------
	// The code below inserts
	//-----------------------------------------------------------------
	if (BridgeTalk.appName == "bridge") {	
		// Use temp function to keep vars out of global namespace.
		drbrownserv.tempFunction = function () {
			// Set up the info necessary for inserting this item into the Bridge's menus later.
			var menuItemInfo = new Object;
			menuItemInfo.text = "Dr. Brown's Auto Correct";
			menuItemInfo.name = 'DrBrownAutoCorrect';
			menuItemInfo.onDisplay = drbrownserv.alwaysEnabled;
			menuItemInfo.onSelect = drbrownserv.autocorrectFromBridge;
			
			drbrownserv.menuItemInfoArray.push (menuItemInfo);
			}
		
		drbrownserv.tempFunction ();
		delete drbrownserv.tempFunction;
		}


	//-----------------------------------------------------------------
	// The code below inserts separater
	//-----------------------------------------------------------------
	if (BridgeTalk.appName == "bridge") {	
		// Use temp function to keep vars out of global namespace.
		drbrownserv.tempFunction = function () {
			// Set up the info necessary for inserting this item into the Bridge's menus later.
			var menuItemInfo = new Object;
			menuItemInfo.text = "-----";
			menuItemInfo.name = '-----';
			//menuItemInfo.onDisplay = drbrownserv.alwaysEnabled;
			//menuItemInfo.onSelect = drbrownserv.captionmakerFromBridge;
			
			drbrownserv.menuItemInfoArray.push (menuItemInfo);
			}
		
		drbrownserv.tempFunction ();
		delete drbrownserv.tempFunction;
		}


	//-----------------------------------------------------------------
	// The code below inserts the Dr. Brown's Merge-A-Matic menu item 
	// into the Bridge menus.
	//-----------------------------------------------------------------
	if (BridgeTalk.appName == "bridge") {
		// Use temp function to keep vars out of global namespace.
		drbrownserv.tempFunction = function () {
			// Set up the info necessary for inserting this item into the Bridge's menus later.
			var menuItemInfo = new Object;
			menuItemInfo.text = "Dr. Brown's Place-A-Matic 16 bit"; 
			menuItemInfo.name = "DrBrownsPlaceAMatic16";
			menuItemInfo.onDisplay = drbrownserv.alwaysEnabled;
			menuItemInfo.onSelect = drbrownserv.drBrownsPlaceAMaticFromBridge16;
		
			drbrownserv.menuItemInfoArray.push (menuItemInfo);
		}
	
		drbrownserv.tempFunction ();
		delete drbrownserv.tempFunction;
	}

	
	//-----------------------------------------------------------------
	// The code below inserts the Dr. Brown's Merge-A-Matic menu item 
	// into the Bridge menus.
	//-----------------------------------------------------------------
	if (BridgeTalk.appName == "bridge") {
		// Use temp function to keep vars out of global namespace.
		drbrownserv.tempFunction = function () {
			// Set up the info necessary for inserting this item into the Bridge's menus later.
			var menuItemInfo = new Object;
			menuItemInfo.text = "Dr. Brown's Place-A-Matic 8 bit"; 
			menuItemInfo.name = "DrBrownsPlaceAMatic8";
			menuItemInfo.onDisplay = drbrownserv.alwaysEnabled;
			menuItemInfo.onSelect = drbrownserv.drBrownsPlaceAMaticFromBridge8;
		
			drbrownserv.menuItemInfoArray.push (menuItemInfo);
		}
	
		drbrownserv.tempFunction ();
		delete drbrownserv.tempFunction;
	}

	
	//-----------------------------------------------------------------
	// The code below inserts the Dr. Brown's Merge-A-Matic menu item 
	// into the Bridge menus.
	//-----------------------------------------------------------------
	if (BridgeTalk.appName == "bridge") {
		// Use temp function to keep vars out of global namespace.
		drbrownserv.tempFunction = function () {
			// Set up the info necessary for inserting this item into the Bridge's menus later.
			var menuItemInfo = new Object;
			menuItemInfo.text = "Dr. Brown's Merge-A-Matic"; 
			menuItemInfo.name = "DrBrownsMergeAMatic";
			menuItemInfo.onDisplay = drbrownserv.alwaysEnabled;
			menuItemInfo.onSelect = drbrownserv.drBrownsMergeAMaticFromBridge;
		
			drbrownserv.menuItemInfoArray.push (menuItemInfo);
		}
	
		drbrownserv.tempFunction ();
		delete drbrownserv.tempFunction;
	}

	
	//-----------------------------------------------------------------
	// The code below inserts separater
	//-----------------------------------------------------------------
	if (BridgeTalk.appName == "bridge") {	
		// Use temp function to keep vars out of global namespace.
		drbrownserv.tempFunction = function () {
			// Set up the info necessary for inserting this item into the Bridge's menus later.
			var menuItemInfo = new Object;
			menuItemInfo.text = "------";
			menuItemInfo.name = 'foo1';
			//menuItemInfo.onDisplay = drbrownserv.alwaysEnabled;
			//menuItemInfo.onSelect = drbrownserv.captionmakerFromBridge;
			
			drbrownserv.menuItemInfoArray.push (menuItemInfo);
			}
		
		drbrownserv.tempFunction ();
		delete drbrownserv.tempFunction;
		}

	//-----------------------------------------------------------------
	// The code below inserts the Dr. Brown's 1-2-3 Process menu item into the
	// Bridge menus.
	//-----------------------------------------------------------------
	if (BridgeTalk.appName == "bridge") {	
		// Use temp function to keep vars out of global namespace.
		drbrownserv.tempFunction = function () {
			// Set up the info necessary for inserting this item into the Bridge's menus later.
			var menuItemInfo = new Object;
			menuItemInfo.text = "Dr. Brown's 1-2-3 Process";
			menuItemInfo.name = 'Process123';
			menuItemInfo.onDisplay = drbrownserv.alwaysEnabled;
			menuItemInfo.onSelect = drbrownserv.process123FromBridge;
			
			drbrownserv.menuItemInfoArray.push (menuItemInfo);
			}
		
		drbrownserv.tempFunction ();
		delete drbrownserv.tempFunction;
		}


	//-----------------------------------------------------------------
	// The code below inserts the Dr. Brown's Caption Maker menu item into the
	// Bridge menus.
	//-----------------------------------------------------------------
	if (BridgeTalk.appName == "bridge") {	
		// Use temp function to keep vars out of global namespace.
		drbrownserv.tempFunction = function () {
			// Set up the info necessary for inserting this item into the Bridge's menus later.
			var menuItemInfo = new Object;
			menuItemInfo.text = "Dr. Brown's Caption Maker";
			menuItemInfo.name = 'CaptionMaker';
			menuItemInfo.onDisplay = drbrownserv.alwaysEnabled;
			menuItemInfo.onSelect = drbrownserv.captionmakerFromBridge;
			
			drbrownserv.menuItemInfoArray.push (menuItemInfo);
			}
		
		drbrownserv.tempFunction ();
		delete drbrownserv.tempFunction;
		}


	//=================================================================
	// Setup Tools > Photoshop menu in Bridge.
	// Creates the menu, then sorts all the menu items we want to
	// create before actually adding them to the menu.
	//=================================================================

	if (BridgeTalk.appName == "bridge") {
	
		// Use temp function to keep vars out of global namespace.
		drbrownserv.tempFunction = function () {
			// The 'Photoshop Services' menu is installed by OLS, the 'Photoshop' menu
			// in installed either by this script, or by the Workflow Automation Scripts
			// (AdobeLibrary1.jsx) which also install menus for other apps. To get the
			// menus in the correct order, all three scripts cooperate - so make sure
			// you are aware of what's happening before changing this code.
			var psServicesMenuExists = MenuElement.find ('Tools/DrBrownsServices') != null;
		
			// Add a new Photoshop submenu in the Bridge's Tools menu.
			drBrownsSubMenu = MenuElement.create (	"menu",
													"Dr. Brown's Services " + drbrownserv.version,
													'-after submenu/VersionCue', 
													'tools/drbr');

			drBrownsSubMenu.onDisplay = drbrownserv.alwaysEnabled;
		
			// Define a function that will sort the menu items for us.
			function menuTextOrder (a, b) {
				// Pass 'a' into 'b' to get reverse sort order so that they can be
				// added at the top of the menu and still be in alphabetical order
				// (the WAS scripts install additional menu items that end up below
				// these).
				return b.text.toLocaleLowerCase ().localeCompare (a.text.toLocaleLowerCase ());
			}
		
			// Sort the menu items
			// drbrownserv.menuItemInfoArray.sort (menuTextOrder);
		
			// Add the menu items to the Photoshop menu.
			for (var index = 0; index < drbrownserv.menuItemInfoArray.length; index++) {
				var menuItemInfo = drbrownserv.menuItemInfoArray[index];

				var menuItem = MenuElement.create (	'command',
													menuItemInfo.text,
													'at the beginning of tools/drbr',
													menuItemInfo.name);
			
				menuItem.onDisplay = menuItemInfo.onDisplay;
				menuItem.onSelect = menuItemInfo.onSelect;	
			}
		
			delete drbrownserv.menuItemInfoArray;
		}
		
		drbrownserv.tempFunction ();
		delete drbrownserv.tempFunction;
	}


	//=================================================================
	// Mini Processor
	// Sets up Dr. Brown's Mini Processor JavaScript to be
	// accessed from the Bridge.
	//=================================================================


	//=================================================================
	// main routine for Mini Processor
	//=================================================================
	drbrownserv.miniprocessor = function (infiles, autoMode, makeFolder, bwMode) {
		try {
			if (BridgeTalk.appName != "photoshop") {
				BridgeTalk.bringToFront ("photoshop");

				var outfiles = drbrownserv.findNewHomeForFiles (infiles, autoMode, makeFolder, bwMode, true); // last param is for making a copy
				var btMessage = new BridgeTalk;
				btMessage.target = "photoshop";
				btMessage.body = "drbrownserv.miniprocessor (" + outfiles.toSource () + "," + autoMode + "," + makeFolder + "," + bwMode + ");";
				btMessage.send ();
			} else {

				try {

					var rememberDialogModes = app.displayDialogs;
					app.displayDialogs = DialogModes.NO;

					var jpegQ = new JPEGSaveOptions();
					jpegQ.quality = 12;

					if (bwMode) {
						try {
							for (var i = 0; i < infiles.length; i++) {
								app.open( infiles[i] );
								var d = app.activeDocument;
								d.flatten();
								d.changeMode( ChangeMode.LAB );
								d.activeChannels = [d.activeChannels[0]];
								d.changeMode( ChangeMode.GRAYSCALE );
								
								// set selection to current channel
								// =======================================================
								var id61 = charIDToTypeID( "setd" );
								var desc15 = new ActionDescriptor();
								var id62 = charIDToTypeID( "null" );
								var ref10 = new ActionReference();
								var id63 = charIDToTypeID( "Chnl" );
								var id64 = charIDToTypeID( "fsel" );
								ref10.putProperty( id63, id64 );
								desc15.putReference( id62, ref10 );
								var id65 = charIDToTypeID( "T   " );
								var ref11 = new ActionReference();
								var id66 = charIDToTypeID( "Chnl" );
								var id67 = charIDToTypeID( "Ordn" );
								var id68 = charIDToTypeID( "Trgt" );
								ref11.putEnumerated( id66, id67, id68 );
								desc15.putReference( id65, ref11 );
								executeAction( id61, desc15, DialogModes.NO );
								desc15 = null;
								ref10 = null;
								ref11 = null;

								d.selection.invert();
								d.changeMode( ChangeMode.RGB );
								d.artLayers.add();
								
								// make new channel at mask channel using reveal selection
								// =======================================================
								var id147 = charIDToTypeID( "Mk  " );
								var desc34 = new ActionDescriptor();
								var id148 = charIDToTypeID( "Nw  " );
								var id149 = charIDToTypeID( "Chnl" );
								desc34.putClass( id148, id149 );
								var id150 = charIDToTypeID( "At  " );
								var ref20 = new ActionReference();
								var id151 = charIDToTypeID( "Chnl" );
								var id152 = charIDToTypeID( "Chnl" );
								var id153 = charIDToTypeID( "Msk " );
								ref20.putEnumerated( id151, id152, id153 );
								desc34.putReference( id150, ref20 );
								var id154 = charIDToTypeID( "Usng" );
								var id155 = charIDToTypeID( "UsrM" );
								var id156 = charIDToTypeID( "RvlS" );
								desc34.putEnumerated( id154, id155, id156 );
								executeAction( id147, desc34, DialogModes.NO );
								desc34 = null;
								ref20 = null;

								// skipping - select current channel with make visible
								// skipping - select current channel
								d.activeLayer.blendMode = BlendMode.MULTIPLY;

								// =======================================================
								var id9 = charIDToTypeID( "slct" );
								var desc3 = new ActionDescriptor();
								var id10 = charIDToTypeID( "null" );
								var ref2 = new ActionReference();
								var id11 = charIDToTypeID( "Chnl" );
								var id12 = charIDToTypeID( "Chnl" );
								var id13 = charIDToTypeID( "RGB " );
								ref2.putEnumerated( id11, id12, id13 );
								desc3.putReference( id10, ref2 );
								executeAction( id9, desc3, DialogModes.NO );
								desc3 = null;
								ref2 = null;

								// fill using color hue 40.9 saturation 61.569 brightness 58.039 opacity 100 mode normal
								// =======================================================
								var id233 = charIDToTypeID( "Fl  " );
								var desc50 = new ActionDescriptor();
								var id234 = charIDToTypeID( "Usng" );
								var id235 = charIDToTypeID( "FlCn" );
								var id236 = charIDToTypeID( "Clr " );
								desc50.putEnumerated( id234, id235, id236 );
								var id237 = charIDToTypeID( "Clr " );
								var desc51 = new ActionDescriptor();
								var id238 = charIDToTypeID( "H   " );
								var id239 = charIDToTypeID( "#Ang" );
								desc51.putUnitDouble( id238, id239, 40.9 );
								var id240 = charIDToTypeID( "Strt" );
								desc51.putDouble( id240, 61.569 );
								var id241 = charIDToTypeID( "Brgh" );
								desc51.putDouble( id241, 58.039 );
								var id242 = charIDToTypeID( "HSBC" );
								desc50.putObject( id237, id242, desc51 );
								var id243 = charIDToTypeID( "Opct" );
								var id244 = charIDToTypeID( "#Prc" );
								desc50.putUnitDouble( id243, id244, 100.000000 );
								var id245 = charIDToTypeID( "Md  " );
								var id246 = charIDToTypeID( "BlnM" );
								var id247 = charIDToTypeID( "Nrml" );
								desc50.putEnumerated( id245, id246, id247 );
								executeAction( id233, desc50, DialogModes.NO );
								desc50 = null;
								desc51 = null;

								// if first doc then pop hue sat dialog and get settings
								if ( i == 0 ) {
									
									// there is no way to tell the difference between 'cancel' and 'ok' with no edits
									// users will need to hit the esc key to actually stop the script
									var hueSatDesc;
									try {
    									var id27 = charIDToTypeID( "HStr" );
									    hueSatDesc = executeAction( id27, undefined, DialogModes.ALL );
									}
									catch( e ) {
									    hueSatDesc = undefined;
									}
								
								} else {
									executeAction( id27, hueSatDesc, DialogModes.NO );
								}

								// really, even on PSD and TIFF and PDF
								d.flatten();
								
								// we need the q factor or we get annoying dialogs
								// check the extension and slide it up to the max for now

								drbrownserv.saveFile( d, jpegQ );

							} // main for loop ends here

							hueSatDesc = null;
							$.gc();

						}
						catch(e) {
							// for debugging only alert(e +":"+ e.line);
						}
					} else {
						var i = 0;
						if (!autoMode) {
							i = 1;
							app.open( infiles[0] );

                            // an ok with no adjustment throws a user canceled error, catch all errors and proceed on
    						// users will need to hit the esc key to actually stop the script
							var levelsDesc;
                            try {
    							var id19 = charIDToTypeID( "Lvls" );
							    levelsDesc = executeAction( id19, undefined, DialogModes.ALL );
							}
							catch( error ) {
							    levelsDesc = undefined;
							}

                            // an ok with no adjustment throws a user canceled error, catch all errors and proceed on
    						// users will need to hit the esc key to actually stop the script
                            var hueSatDesc;
							try {
    							var id27 = charIDToTypeID( "HStr" );
    							hueSatDesc = executeAction( id27, undefined, DialogModes.ALL );
    						}
    						catch( error ) {
    						    hueSatDesc = undefined;
    						}

							drbrownserv.saveFile( app.activeDocument, jpegQ );
						}
						for (/* nothing here on purpose see above for i init */; i < infiles.length; i++) {
							app.open( infiles[i] );
							if (autoMode) {
                                
                                // This is auto color
								var id3 = charIDToTypeID( "Lvls" );
								var desc2 = new ActionDescriptor();
								var id4 = stringIDToTypeID( "autoBlackWhite" );
								desc2.putBoolean( id4, true );
								var id5 = stringIDToTypeID( "autoNeutrals" );
								desc2.putBoolean( id5, true );
								executeAction( id3, desc2, DialogModes.NO );
								desc2 = null;

								app.activeDocument.activeLayer.autoContrast();

							} else {
								executeAction( id19, levelsDesc, DialogModes.NO );
								executeAction( id27, hueSatDesc, DialogModes.NO );
							}
							drbrownserv.saveFile( app.activeDocument, jpegQ );
						}	
					}
					app.displayDialogs = rememberDialogModes;
					$.gc();
				}
				catch(error) {
        			if (error.number != 8007) { // Don't report user cancelled errors.
		    			alert(error + ":" + error.line);
		    		}
				}
			}
		}
		catch (error) {
			if (error.number != 8007) { // Don't report user cancelled errors.
				alert (error + ":" + error.line);
			}
		}
	}

} // Overall try catch

catch (error) {
	// Debugging
	alert (error + ":" + error.line);
}
