// 05/04/2006 08:47
// c2006 Adobe Systems, Inc. All rights reserved.
// Produced and Directed by Dr. Brown ( a.k.a Russell Preston Brown )
// Written by Tom Ruark
// UI Design by Julie Meridian

// JPEG 1-2-3 is copied from Image Processor
// For this version of JPEG 1-2-3 I made some major changes
// 1. The dialog is 95% one big resource string. This makes it easier to change things around.
//    Russell likes to change things around during developement.
// 2. I made it jpeg1, jpeg2, jpeg3 to make everythig more clear.

// enable double clicking from the Macintosh Finder or the Windows Explorer
#target photoshop

// in case we double clicked the file
app.bringToFront();

// debug level: 0-2 (0:disable, 1:break on error, 2:break at beginning)
// $.level = 1;
// debugger; // launch debugger on next line

// on localized builds we pull the $$$/Strings from a .dat file, see documentation for more details
$.localize = true;

// the main routine
// the ImageProcessor object does most of the work
try { 

	GlobalVariables();

	CheckVersion();
	
	if ( IsWindowsOS() ) {
		gShortFileNameLength = gShortFileNameLengthWin;
	}

	var gIP = new ImageProcessor();

	gIP.LoadParamsFromDisk( GetDefaultParamsFile() );

	gIP.CreateDialog();
	
	if ( gIP.RunDialog() ) {

		gIP.Execute();
		
		gIP.SaveParamsToDisk( GetDefaultParamsFile() );
	}

	$.gc();
}

// Lot's of things can go wrong
// Give a generic alert and see if they want the details
catch( e ) {
	if ( confirm( strSorry ) ) {
		alert( e + " : " + e.line );
	}
}

// restore the units
preferences.rulerUnits = gStartRulerUnits;

// restore the dialog modes
app.displayDialogs = gSaveDialogMode;


//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////



///////////////////////////////////////////////////////////////////////////////
// Function: GlobalVariables
// Usage: all of my globals
// Input: <none>
// Return: <none>
///////////////////////////////////////////////////////////////////////////////
function GlobalVariables() {

	// a version for possible expansion issues
	gVersion = 1;
	
	// A list of file extensions I will skip, keep them lower case
	gFilesToSkip = Array( "db", "xmp", "thm", "txt", "doc", "md0", "tb0", "adobebridgedb", "adobebridgedbt", "bc", "bct", "atn" );

	// A list of camera raw extensions, keep them lower case
	gFilesForCameraRaw = Array( "tif", "crw", "nef", "raf", "orf", "mrw", "dcr", "mos", "srf", "pef", "dcr", "cr2", "dng", "erf", "x3f", "raw" );

	// limit the length of text in edit boxes by this length
	gShortFileNameLengthWin = 30;
	gShortFileNameLengthMac = 26;
	gShortFileNameLength = gShortFileNameLengthMac;

	// remember the dialog modes
	gSaveDialogMode = app.displayDialogs;
	app.displayDialogs = DialogModes.NO;

    // remember the units
    gStartRulerUnits = preferences.rulerUnits;
	preferences.rulerUnits = Units.PIXELS;

	gClassActionSet = charIDToTypeID( 'ASet' );
	gClassAction = charIDToTypeID( 'Actn' );
	gKeyName = charIDToTypeID( 'Nm  ' );
	gKeyNumberOfChildren = charIDToTypeID( 'NmbC' );

	gStrTitle = "Dr. Brown's 1-2-3 JPEG";

    // I pulled a lot of the localization strings to make the dialog resource easier to read and manage	
	strNoImagesSelected = localize( "$$$/JavaScripts/ImageProcessor/NoImagesSelected=No images have been selected" );
	strNoFolderSelected = localize( "$$$/JavaScripts/ImageProcessor/NoFolderSelected=No folder has been selected" );
	strPickXML = localize( "$$$/JavaScripts/ImageProcessor/PickXML=Pick a XML file to load" );
	strPickXMLWin = localize( "$$$/JavaScripts/ImageProcessor/PickXMLWin=XML Files: *.xml" );
	strPickXMLSave = localize( "$$$/JavaScripts/ImageProcessor/PickXMLSave=Pick a XML file to save" );
	strPickSource = localize( "$$$/JavaScripts/ImageProcessor/PickSource=Pick a source folder" );
	strPickDest = localize( "$$$/JavaScripts/ImageProcessor/PickDest=Pick a destination folder" );
	strSpecifySource = localize( "$$$/JavaScripts/ImageProcessor/SpecifySource=Please specify a source folder." );
	strSpecifyDest = localize( "$$$/JavaScripts/ImageProcessor/SpecifyDest=Please specify a destination folder." );
	strJPEGQuality = localize( "$$$/JavaScripts/ImageProcessor/JPEGQuality=JPEG Quality must be between 0 and 12." );
	strJPEGWandH = localize( "$$$/JavaScripts/ImageProcessor/JPEGWandH=You must specify width and/or height when using resize image options for JPEG." );
	strOneType = localize( "$$$/JavaScripts/ImageProcessor/OneType=You must save to at least one file type." );
	strNotaRAW = localize( "$$$/JavaScripts/ImageProcessor/NotARAW=This is not a camera raw file according to my list of extensions, " );
	strWidthAndHeight = localize( "$$$/JavaScripts/ImageProcessor/WidthAndHeight=Width and Height must be defined to use FitImage function!" );
	strMustUse = localize( "$$$/JavaScripts/ImageProcessor/MustUse=You must use Photoshop CS 2 or later to run this script!" );
	strSorry = localize( "$$$/JavaScripts/ImageProcessor/Sorry=Sorry, something major happened and I can't continue! Would you like to see more info?" );
	strMustSaveOpen = localize( "$$$/JavaScripts/ImageProcessor/MustSaveOpen=Open files must be saved before they can be used by the Image Processor." );
	strFollowing = localize( "$$$/JavaScripts/ImageProcessor/Following=The following files will not be saved." );
}

// given a file name and a list of extensions
// determine if this file is in the list of extensions
function IsFileOneOfThese( inFileName, inArrayOfFileExtensions ) {
	var lastDot = inFileName.toString().lastIndexOf( "." );
	if ( lastDot == -1 ) {
		return false;
	}
	var strLength = inFileName.toString().length;
	var extension = inFileName.toString().substr( lastDot + 1, strLength - lastDot );
	extension = extension.toLowerCase();
	for (var i = 0; i < inArrayOfFileExtensions.length; i++ ) {
		if ( extension == inArrayOfFileExtensions[i] ) {
			return true;
		}
	}
	return false;
}

// the main class
function ImageProcessor() {

	// load my params from the xml file on disk if it exists
	// this.params["myoptionname"] = myoptionvalue
	// I wrote a very simple xml parser, I'm sure it needs work
	this.LoadParamsFromDisk = function( loadFile ) {
		if ( loadFile.exists ) {
			loadFile.open( "r" );
			var projectSpace = ReadHeader( loadFile );
			if ( projectSpace == GetScriptNameForXML() ) {
				while ( ! loadFile.eof ) {
					var starter = ReadHeader( loadFile );
					var data = ReadData( loadFile );
					var ender = ReadHeader( loadFile );
					if ( ( "/" + starter ) == ender ) {
						this.params[starter] = data;
					}
					// force boolean values to boolean types
					if ( data == "true" || data == "false" ) {
						this.params[starter] = data == "true";
					}
				}
			}
			loadFile.close();
			if ( this.params["version"] != gVersion ) {
				// do something here to fix version conflicts
				// this should do it
				this.params["version"] = gVersion;
			}
		} 
	}

	// save out my params, this is much easier
	this.SaveParamsToDisk = function( saveFile ) {
		saveFile.encoding = "UTF8";
		saveFile.open( "w", "TEXT", "????" );
		// unicode signature, this is UTF16 but will convert to UTF8 "EF BB BF"
		saveFile.write("\uFEFF"); 
		var scriptNameForXML = GetScriptNameForXML();
		saveFile.writeln( "<" + scriptNameForXML + ">" );
		for ( var p in this.params ) {
			saveFile.writeln( "\t<" + p + ">" + this.params[p] + "</" + p + ">" );
		}
		saveFile.writeln( "</" + scriptNameForXML + ">" );
		saveFile.close();
	}

	this.CreateDialog = function() {
	
		var dlgResource = "dialog { orientation: 'column', alignChildren: 'fill', spacing: 5, \
		                    grpTop: Group { \
		                      grpLeft: Group { orientation: 'column', alignChildren: 'fill', spacing: 5, \
		                        grp1: Group { orientation: 'row', alignChildren: 'center', \
		                          icnOne: Image { icon: 'Step1Icon', helpTip: 'Select the images to process' }, \
                                  stSourceLabel: StaticText { text: 'Select the images to process', helpTip: 'Select the images to process' } \
		                        }, \
		                        grp1Info: Group { orientation: 'row', alignChildren: 'fill', margins: [ 15, 0, 0, 0 ], \
		                          grpLeft: Group { orientation: 'row', alignChildren: 'fill', \
		                            icnSource: Image { icon: 'SourceFolderIcon', helpTip: 'Select the images to process' } \
		                          }, \
		                          grpRight: Group { orientation: 'column', alignChildren: 'left' } \
		                        }, \
		                        line1: Panel { bounds: [ 0, 0, 0, 0], properties: { borderStyle: 'sunken' } }, \
		                        grp2: Group { orientation: 'row', alignChildren: 'center', spacing: 1, \
		                          icnTwo: Image { icon: 'Step2Icon', helpTip: 'Select location to save processed images' }, \
		                          stDestination: StaticText { text: 'Select location to save processed images', helpTip: 'Select location to save processed images', properties: { idString: 'stDestination' } } \
		                        }, \
		                        grp2Info: Group { orientation: 'row', alignChildren: 'fill', margins: [ 15, 0, 0, 0 ], \
		                          grpLeft: Group { orientation: 'row', alignChildren: 'left', \
		                            icnDest: Image { icon: 'DestinationFolderIcon', helpTip: 'Select location to save processed images' } \
		                          }, \
		                          grpRight: Group { orientation: 'column', alignChildren: 'left', \
		                            grpSaveIn: Group { orientation: 'row', \
		                              rbSaveInSame: RadioButton { text: 'S&ave in Same Location', helpTip: 'Save the new documents next to the original documents', properties: { idString: 'rbSaveInSame' } }, \
		                            }, \
		                            grpDestBrowse: Group { orientation: 'row', alignChildren: 'center', \
		                              rbSaveInNew: RadioButton { helpTip: 'Select location to save processed images', properties: { idString: 'rbSaveInNew' } }, \
		                              btnDest: Button { text: 'Sele&ct Folder...', helpTip: 'Select location to save processed images', properties: { idString: 'btnDest' } }, \
		                              stDest: StaticText { text: 'No folder has been selected', helpTip: 'Select location to save processed images', properties: { idString: 'stDest' } } \
		                            }, \
		                            cbMakeFolders: Checkbox { text: 'Save in sub-folder', helpTip: 'Create a sub folder for output files (JPEG1, JPEG2, JPEG3)', properties: { idString: 'cbMakeFolders' } }, \
		                            grpFolderNames: Group { orientation: 'row', alignChildren: 'center', \
		                              icn1: Image { icon: 'Step1Icon', helpTip: 'Enter the name for the first sub folder', properties: { idString: 'icn1' }  }, \
		                              etFolder1: EditText { text: 'Small', properties: { idString: 'etFolder1' } }, \
		                              icn2: Image { icon: 'Step2Icon', helpTip: 'Enter the name for the second sub folder', properties: { idString: 'icn2' }  }, \
		                              etFolder2: EditText { text: 'Medium', properties: { idString: 'etFolder2' } }, \
		                              icn3: Image { icon: 'Step3Icon', helpTip: 'Enter the name for the third sub folder', properties: { idString: 'icn3' }  }, \
		                              etFolder3: EditText { text: 'Large', properties: { idString: 'etFolder3' } }, \
		                            } \
		                          } \
		                        } \
  		                      }, \
  		                      grpRight: Group { orientation: orientation: 'column', alignChildren: 'right', \
							    btnRun: Button { text: 'Run', properties: { name: 'ok', idString: 'btnRun' } }, \
                                btnCancel: Button { text: 'Cancel', properties: { name: 'cancel', idString: 'btnCancel' } }, \
								stFake: StaticText { }, \
                                btnLoad: Button { text: '&Load...', helpTip: 'Load a settings file from disk', properties: { idString: 'btnLoad' } }, \
                                btnSave: Button { text: '&Save...', helpTip: 'Save the current dialog settings to disk', properties: { idString: 'btnSave' } } \
		                      } \
		                    }, \
		                    line2: Panel { bounds: [ 0, 0, 0, 0], properties: { borderStyle: 'sunken' } }, \
		                    grp3: Group { orientation: 'row', alignChildren: 'center', spacing: 1, \
		                      icnThree: Image { icon: 'Step3Icon', helpTip: 'JPEG File Types' }, \
		                      stFileType: StaticText { text: 'JPEG File Types', helpTip: 'JPEG File Types' } \
		                    }, \
		                    grp3Info: Group { orientation: 'row', alignChildren: 'fill', margins: [ 15, 0, 0, 0 ], \
		                      grpLeft: Group { orientation: 'column', alignChildren: 'left', \
		                        icnSpace: Image { icon: 'DestinationFolderIcon', visible: false } \
		                      }, \
		                      grpRight: Group { orientation: 'column', alignChildren: 'left', \
		                        grpFileType: Group { orientation: 'column', alignChildren: 'fill', \
		                          grpFile1: Group { orientation: 'row', alignChildren: 'fill', \
		                            grpLeft: Group { orientation: 'column', alignChildren: 'fill', \
		                              cbJPEG: Checkbox { text: 'Save as &JPEG 1', helpTip: 'Save a file to the JPEG format', properties: { idString: 'cbJPEG1' } }, \
		                              grpJPEG: Group { orientation: 'column', alignChildren: 'left', margins: [ 15, 0, 0, 0], \
		                                grpQ: Group { orientation: 'row', alignChildren: 'center', \
		                                  stQuality: StaticText { text: 'Quality:', properties: { idString: 'stQuality1' } }, \
		                                  etQuality: EditText { properties: { idString: 'etQuality1' } } \
		                                }, \
		                                cbConvertICC: Checkbox { text: 'Convert Profile to sRGB', helpTip: 'Convert the ICC profile to sRGB before saving', properties: { idString: 'cbConvertICC1' } }, \
		                                cbRunAction: Checkbox { text: 'Run Action', helpTip: 'Run the action for this save file?', properties: { idString: 'cbRunAction1' } }, \
		                              } \
		                            }, \
   	                                hidden6: EditText { visible: false }, \
		                            grpRight: Group { orientation: 'column', alignChildren: 'center', \
		                              grpTop: Group { orientation: 'column', alignChildren: 'center', \
		                                cbResizeJPEG: Checkbox { text: '&Resize to Fit', helpTip: 'Select to resize for this format', properties: { idString: 'cbResizeJPEG1' } }, \
		                              }, \
		                              grpBottom: Group { orientation: 'row', \
		                                grpSt: Group { orientation: 'column', alignChildren: 'right', spacing: 16, \
		                                  stResizeJPEGW: StaticText { text: 'W:', helpTip: 'Type in a width to resize image', properties: { idString: 'stResizeJPEGW1' } }, \
		                                  stResizeJPEGH: StaticText { text: 'H:', helpTip: 'Type in a height to resize image', properties: { idString: 'stResizeJPEGH1' } }, \
		                                  stResizeJPEGR: StaticText { text: 'Resolution:', helpTip: 'Type in the resolution of the final image', properties: { idString: 'stResizeJPEGR1' } }, \
		                                }, \
		                                grpEt: Group { orientation: 'column', alignChildren: 'fill', spacing: 8, \
		                                  etResizeJPEGW: EditText { properties: { idString: 'etResizeJPEGW1' } }, \
		                                  etResizeJPEGH: EditText { properties: { idString: 'etResizeJPEGH1' } }, \
		                                  etResizeJPEGR: EditText { properties: { idString: 'etResizeJPEGR1' }  }, \
		                                }, \
		                                grpDd: Group { orientation: 'column', alignChildren: 'fill', spacing: 5, \
		                                  ddResizeJPEGW: DropDownList { properties: { idString: 'ddResizeJPEGW1', items: [ 'pixels', 'percent', 'inches', 'cm', 'mm', 'points', 'picas' ] } }, \
		                                  ddResizeJPEGH: DropDownList { properties: { idString: 'ddResizeJPEGH1', items: [ 'pixels', 'percent', 'inches', 'cm', 'mm', 'points', 'picas' ] } }, \
		                                  ddResizeJPEGR: DropDownList { properties: { idString: 'ddResizeJPEGR1', items: [ 'pixels/inch', 'pixels/cm' ] } }, \
		                                } \
		                              } \
		                            } \
		                          }, \
		                          line3: Panel { bounds: [ 0, 0, 0, 0], properties: { borderStyle: 'sunken' } }, \
		                          grpFile2: Group { orientation: 'row', alignChildren: 'fill', \
		                            grpLeft: Group { orientation: 'column', alignChildren: 'fill', \
		                              cbJPEG: Checkbox { text: 'Save as &JPEG 2', helpTip: 'Save a file to the JPEG format', properties: { idString: 'cbJPEG2' } }, \
		                              grpJPEG: Group { orientation: 'column', alignChildren: 'left', margins: [ 15, 0, 0, 0], \
		                                grpQ: Group { orientation: 'row', alignChildren: 'center', \
		                                  stQuality: StaticText { text: 'Quality:', properties: { idString: 'stQuality2' } }, \
		                                  etQuality: EditText { properties: { idString: 'etQuality2' } } \
		                                }, \
		                                cbConvertICC: Checkbox { text: 'Convert Profile to sRGB', helpTip: 'Convert the ICC profile to sRGB before saving', properties: { idString: 'cbConvertICC2' } }, \
		                                cbRunAction: Checkbox { text: 'Run Action', helpTip: 'Run the action for this save file?', properties: { idString: 'cbRunAction2' } }, \
		                              } \
		                            }, \
   	                                hidden6: EditText { visible: false }, \
		                            grpRight: Group { orientation: 'column', alignChildren: 'center', \
		                              grpTop: Group { orientation: 'column', alignChildren: 'center', \
		                                cbResizeJPEG: Checkbox { text: '&Resize to Fit', helpTip: 'Select to resize for this format', properties: { idString: 'cbResizeJPEG2' } }, \
		                              }, \
		                              grpBottom: Group { orientation: 'row', \
		                                grpSt: Group { orientation: 'column', alignChildren: 'right', spacing: 16, \
		                                  stResizeJPEGW: StaticText { text: 'W:', helpTip: 'Type in a width to resize image', properties: { idString: 'stResizeJPEGW2' } }, \
		                                  stResizeJPEGH: StaticText { text: 'H:', helpTip: 'Type in a height to resize image', properties: { idString: 'stResizeJPEGH2' } }, \
		                                  stResizeJPEGR: StaticText { text: 'Resolution:', helpTip: 'Type in the resolution of the final image', properties: { idString: 'stResizeJPEGR2' } }, \
		                                }, \
		                                grpEt: Group { orientation: 'column', alignChildren: 'fill', spacing: 8, \
		                                  etResizeJPEGW: EditText { properties: { idString: 'etResizeJPEGW2' } }, \
		                                  etResizeJPEGH: EditText { properties: { idString: 'etResizeJPEGH2' } }, \
		                                  etResizeJPEGR: EditText { properties: { idString: 'etResizeJPEGR2' }  }, \
		                                }, \
		                                grpDd: Group { orientation: 'column', alignChildren: 'fill', spacing: 5, \
		                                  ddResizeJPEGW: DropDownList { properties: { idString: 'ddResizeJPEGW2', items: [ 'pixels', 'percent', 'inches', 'cm', 'mm', 'points', 'picas' ] } }, \
		                                  ddResizeJPEGH: DropDownList { properties: { idString: 'ddResizeJPEGH2', items: [ 'pixels', 'percent', 'inches', 'cm', 'mm', 'points', 'picas' ] } }, \
		                                  ddResizeJPEGR: DropDownList { properties: { idString: 'ddResizeJPEGR2', items: [ 'pixels/inch', 'pixels/cm' ] } }, \
		                                } \
		                              } \
		                            } \
		                          }, \
		                          line4: Panel { bounds: [ 0, 0, 0, 0], properties: { borderStyle: 'sunken' } }, \
		                          grpFile3: Group { orientation: 'row', alignChildren: 'fill', \
		                            grpLeft: Group { orientation: 'column', alignChildren: 'fill', \
		                              cbJPEG: Checkbox { text: 'Save as &JPEG 3', helpTip: 'Save a file to the JPEG format', properties: { idString: 'cbJPEG3' } }, \
		                              grpJPEG: Group { orientation: 'column', alignChildren: 'left', margins: [ 15, 0, 0, 0], \
		                                grpQ: Group { orientation: 'row', alignChildren: 'center', \
		                                  stQuality: StaticText { text: 'Quality:', properties: { idString: 'stQuality3' } }, \
		                                  etQuality: EditText { properties: { idString: 'etQuality3' }} \
		                                }, \
		                                cbConvertICC: Checkbox { text: 'Convert Profile to sRGB', helpTip: 'Convert the ICC profile to sRGB before saving', properties: { idString: 'cbConvertICC3' } }, \
		                                cbRunAction: Checkbox { text: 'Run Action', helpTip: 'Run the action for this save file?', properties: { idString: 'cbRunAction3' } }, \
		                              } \
		                            }, \
   	                                hidden6: EditText { visible: false }, \
		                            grpRight: Group { orientation: 'column', alignChildren: 'center', \
		                              grpTop: Group { orientation: 'column', alignChildren: 'center', \
		                                cbResizeJPEG: Checkbox { text: '&Resize to Fit', helpTip: 'Select to resize for this format', properties: { idString: 'cbResizeJPEG3' } }, \
		                              }, \
		                              grpBottom: Group { orientation: 'row', \
		                                grpSt: Group { orientation: 'column', alignChildren: 'right', spacing: 16, \
		                                  stResizeJPEGW: StaticText { text: 'W:', helpTip: 'Type in a width to resize image', properties: { idString: 'stResizeJPEGW3' } }, \
		                                  stResizeJPEGH: StaticText { text: 'H:', helpTip: 'Type in a height to resize image', properties: { idString: 'stResizeJPEGH3' } }, \
		                                  stResizeJPEGR: StaticText { text: 'Resolution:', helpTip: 'Type in the resolution of the final image', properties: { idString: 'stResizeJPEGR3' } }, \
		                                }, \
		                                grpEt: Group { orientation: 'column', alignChildren: 'fill', spacing: 8, \
		                                  etResizeJPEGW: EditText { properties: { idString: 'etResizeJPEGW3' } }, \
		                                  etResizeJPEGH: EditText { properties: { idString: 'etResizeJPEGH3' }  }, \
		                                  etResizeJPEGR: EditText { properties: { idString: 'etResizeJPEGR3' }  }, \
		                                }, \
		                                grpDd: Group { orientation: 'column', alignChildren: 'fill', spacing: 5, \
		                                  ddResizeJPEGW: DropDownList { properties: { idString: 'ddResizeJPEGW3', items: [ 'pixels', 'percent', 'inches', 'cm', 'mm', 'points', 'picas' ] } }, \
		                                  ddResizeJPEGH: DropDownList { properties: { idString: 'ddResizeJPEGH3', items: [ 'pixels', 'percent', 'inches', 'cm', 'mm', 'points', 'picas' ] } }, \
		                                  ddResizeJPEGR: DropDownList { properties: { idString: 'ddResizeJPEGR3', items: [ 'pixels/inch', 'pixels/cm' ] } }, \
		                                } \
		                              } \
		                            } \
		                          } \
		                        } \
		                      }, \
		                    }, \
		                    line5: Panel { bounds: [ 0, 0, 0, 0], properties: { borderStyle: 'sunken' } }, \
		                    grp4: Group { orientation: 'row', alignChildren: 'top', spacing: 1, \
		                      icnFour: Image { icon: 'Step4Icon', helpTip: 'Preferences' }, \
		                      stFileType: StaticText { text: 'Preferences', helpTip: 'Preferences' } \
		                    }, \
		                    grp4Info: Group { orientation: 'column', alignChildren: 'left', margins: [ 15, 0, 0, 0 ], \
		                      grpTop: Group { orientation: 'row', alignChildren: 'top', spacing: 1, \
		                        grpLeft: Group { orientation: 'column', alignChildren: 'left', spacing: 1, \
		                          grpAction: Group { orientation: 'row', \
		                            stAction: StaticText { text: 'Select Action:', helpTip: 'Select an action set and an action' }, \
		                            hidden4: DropDownList { visible: false } \
		                          }, \
		                          grpCopyright: Group { orientation: 'row', \
		                            stCopyrightInfo: StaticText { text: 'Copyright Info:', helpTip: 'Add text to copyright your images' }, \
		                            hidden5: EditText { visible: false } \
		                          }, \
		                        }, \
		                        grpRight: Group { orientation: 'column', alignChildren: 'left', spacing: 1, \
                                  grpDropDowns: Group { orientation: 'row', alignChildren: 'center', \
                                    ddSet: DropDownList { helpTip: 'Select an action set and an action', properties: { idString: 'ddSet' } }, \
                                    ddAction: DropDownList { helpTip: 'Select an action set and an action', properties: { idString: 'ddAction' } } \
		                          }, \
		                          etCopyrightInfo: EditText { helpTip: 'Add text to copyright your images', alignment: 'fill', properties: { idString: 'etCopyrightInfo' } } \
		                        }, \
		                      }, \
	                          grpICC: Group { orientation: 'row', alignChildren: 'center', spacing: 1, \
                                cbIncludeICC: Checkbox { text: 'Inclu&de ICC Profile', helpTip: 'Include the ICC Profile when saving the file', properties: { idString: 'cbIncludeICC' } }, \
	                            hidden6: EditText { visible: false } \
	                          }, \
                            }, \
		                  }";
		                

        this.dlgMain = new Window( dlgResource );
        
        this.dlgMain.initisover1 = false;
        this.dlgMain.initisover2 = false;
        this.dlgMain.initisover3 = false;

		// some interesting numbers to help the auto layout
		var wAndhEtLength = 40;
		var folderEtLength = 55;
		var qEtLength = 30;
		var bridgeLength = 300;
		 
	 	if ( IsWindowsOS() ) {
		    var sourceLength = 160;
		    var actionDdLength = 150;
		} else {
		    var sourceLength = 200;
		    var actionDdLength = 175;
		}
		
        // some runtime creation
        if ( this.runningFromBridge ) {
            this.dlgMain.stBridge = this.dlgMain.grpTop.grpLeft.grp1Info.grpRight.add( 'statictext', undefined, "Process files from Bridge only (" + this.filesFromBridge.length + ")" );
            this.dlgMain.stBridge.helpTip = 'Selected files from Bridge will be processed';
            this.dlgMain.stBridge.preferredSize.width = bridgeLength;
        } else {
	        this.dlgMain.rbUseOpen = this.dlgMain.grpTop.grpLeft.grp1Info.grpRight.add( 'radiobutton', undefined, 'Use Open &Images' );
	        this.dlgMain.rbUseOpen.helpTip = 'Use all open images';
	        var grpSelect = this.dlgMain.grpTop.grpLeft.grp1Info.grpRight.add( 'group', undefined, { orientation: 'row', alignChildren: 'fill' } );
	        this.dlgMain.rbUseFolder = grpSelect.add( 'radiobutton', undefined, '');
	        this.dlgMain.rbUseFolder.helpTip = 'Use the images that are currently open';
	        this.dlgMain.btnSource = grpSelect.add( 'button', undefined, 'Select &Folder...' );
	        this.dlgMain.btnSource.helpTip = 'Select the images to process';
	        this.dlgMain.stSource = grpSelect.add( 'statictext', undefined, 'No images have been selected' );
	        this.dlgMain.stSource.helpTip = 'Location of files to process';
	        this.dlgMain.stSource.preferredSize.width = sourceLength;
        }

        this.dlgMain.text = gStrTitle;

        FlattenDialog( this.dlgMain, this.dlgMain );

        this.dlgMain.etFolder1.preferredSize.width = folderEtLength;
        this.dlgMain.etFolder2.preferredSize.width = folderEtLength;
        this.dlgMain.etFolder3.preferredSize.width = folderEtLength;

        this.dlgMain.etQuality1.preferredSize.width = qEtLength;
        this.dlgMain.etQuality2.preferredSize.width = qEtLength;
        this.dlgMain.etQuality3.preferredSize.width = qEtLength;

        this.dlgMain.etResizeJPEGW1.preferredSize.width = wAndhEtLength;
        this.dlgMain.etResizeJPEGW2.preferredSize.width = wAndhEtLength;
        this.dlgMain.etResizeJPEGW3.preferredSize.width = wAndhEtLength;
        this.dlgMain.etResizeJPEGH1.preferredSize.width = wAndhEtLength;
        this.dlgMain.etResizeJPEGH2.preferredSize.width = wAndhEtLength;
        this.dlgMain.etResizeJPEGH3.preferredSize.width = wAndhEtLength;
        this.dlgMain.etResizeJPEGR1.preferredSize.width = wAndhEtLength;
        this.dlgMain.etResizeJPEGR2.preferredSize.width = wAndhEtLength;
        this.dlgMain.etResizeJPEGR3.preferredSize.width = wAndhEtLength;

        this.dlgMain.ddSet.preferredSize.width = actionDdLength;
        this.dlgMain.ddAction.preferredSize.width = actionDdLength;
        
        this.dlgMain.etCopyrightInfo.preferredSize.width = actionDdLength + actionDdLength;
        
	}

	this.InitParams = function() {
		var params = new Object();
		params["version"] = gVersion;
		params["useopen"] = false;
		params["source"] = "";
		params["saveinsame"] = true;
		params["makefolders"] = true;
		params["folder1"] = "Small";
		params["folder2"] = "Medium";
		params["folder3"] = "Large";
		params["dest"] = "";
		params["jpeg1"] = true;
		params["jpeg2"] = false;
		params["jpeg3"] = false;
		params["q1"] = 8;
		params["q2"] = 8;
		params["q3"] = 8;
		params["converticc1"] = false;
		params["converticc2"] = false;
		params["converticc3"] = false;
		params["jpegresize1"] = false;
		params["jpegw1"] = "";
		params["jpegh1"] = "";
		params["jpegr1"] = 72;
		params["jpegwu1"] = "pixels";
		params["jpeghu1"] = "pixels";
		params["jpegru1"] = "pixels/inch";
		params["jpegresize2"] = false;
		params["jpegw2"] = "";
		params["jpegh2"] = "";
		params["jpegr2"] = 72;
		params["jpegwu2"] = "pixels";
		params["jpeghu2"] = "pixels";
		params["jpegru2"] = "pixels/inch";
		params["jpegresize3"] = false;
		params["jpegw3"] = "";
		params["jpegh3"] = "";
		params["jpegr3"] = 72;
		params["jpegwu3"] = "pixels";
		params["jpeghu3"] = "pixels";
		params["jpegru3"] = "pixels/inch";
		params["runaction1"] = false;
		params["runaction2"] = false;
		params["runaction3"] = false;
		params["actionset"] = "";
		params["action"] = "";
		params["info"] = "";
		params["icc"] = true;

		return params;
	}

	// transfer from the default settings or settings I read off disk to the dialog widgets
	this.InitDialog = function() {

		var d = this.dlgMain;
		var p = this.params;
		
		this.dlgMain.ip = this;
		d.loadFromDisk = true;
		
		if ( ! this.runningFromBridge ) {

			if ( app.documents.length > 0 ) {
				d.rbUseOpen.value = p["useopen"];
			} else {
				d.rbUseOpen.enabled = false;
				d.rbUseOpen.value = false;
			}
			
			d.rbUseFolder.value = ! d.rbUseOpen.value;

			d.stSource.text = CreateShortFileName( p["source"] );
			if ( d.stSource.text == "" ) {
				d.stSource.text = strNoImagesSelected;
			}
		}
		d.sourceLongText = p["source"];

		d.rbSaveInSame.value = p["saveinsame"];
		d.rbSaveInNew.value = ! d.rbSaveInSame.value;		
		d.stDest.text = CreateShortFileName( p["dest"] );
		if ( d.stDest.text == "" ) {
			d.stDest.text = strNoFolderSelected;
		}
		d.destLongText = p["dest"];
		d.cbMakeFolders.value = p["makefolders"];
		d.etFolder1.text = p["folder1"];
		d.etFolder2.text = p["folder2"];
		d.etFolder3.text = p["folder3"];
		d.cbJPEG1.value = p["jpeg1"];
		d.cbJPEG2.value = p["jpeg2"];
		d.cbJPEG3.value = p["jpeg3"];
		d.cbConvertICC1.value = p["converticc1"];
		d.cbConvertICC2.value = p["converticc2"];
		d.cbConvertICC3.value = p["converticc3"];
		d.etQuality1.text = p["q1"];
		d.etQuality2.text = p["q2"];
		d.etQuality3.text = p["q3"];
		d.cbRunAction1.value = p["runaction1"];
		d.cbRunAction2.value = p["runaction2"];
		d.cbRunAction3.value = p["runaction3"];
		d.cbResizeJPEG1.value = p["jpegresize1"];
		d.etResizeJPEGH1.text = p["jpegh1"];
		d.etResizeJPEGW1.text = p["jpegw1"];
		d.etResizeJPEGR1.text = p["jpegr1"];
		d.cbResizeJPEG2.value = p["jpegresize2"];
		d.etResizeJPEGH2.text = p["jpegh2"];
		d.etResizeJPEGW2.text = p["jpegw2"];
		d.etResizeJPEGR2.text = p["jpegr2"];
		d.cbResizeJPEG3.value = p["jpegresize3"];
		d.etResizeJPEGH3.text = p["jpegh3"];
		d.etResizeJPEGW3.text = p["jpegw3"];
		d.etResizeJPEGR3.text = p["jpegr3"];
		d.etCopyrightInfo.text = p["info"];
		d.cbIncludeICC.value = p["icc"];
	}

	// pretend like i clicked it to get the other items to respond to the current settings
	this.ForceDialogUpdate = function() {	
		if ( ! this.runningFromBridge ) {
			this.dlgMain.rbUseOpen.onClick();
		}
		this.dlgMain.rbSaveInSame.onClick();
		this.dlgMain.cbMakeFolders.onClick();
		this.dlgMain.cbJPEG1.onClick();
		this.dlgMain.cbJPEG2.onClick();
		this.dlgMain.cbJPEG3.onClick();
		this.dlgMain.cbResizeJPEG1.onClick();
		this.dlgMain.cbResizeJPEG2.onClick();
		this.dlgMain.cbResizeJPEG3.onClick();
		index = this.dlgMain.SetDropDown( this.dlgMain.ddSet, this.params["actionset"] );
		if ( this.actionInfo.length > 0 ) {
			this.dlgMain.ddSet.onChange( index );
			this.actionIndex = this.dlgMain.SetDropDown( this.dlgMain.ddAction, this.params["action"] );
		}
		this.dlgMain.SetDropDown( this.dlgMain.ddResizeJPEGW1, this.params["jpegwu1"] );
		this.dlgMain.SetDropDown( this.dlgMain.ddResizeJPEGH1, this.params["jpeghu1"] );
		this.dlgMain.SetDropDown( this.dlgMain.ddResizeJPEGR1, this.params["jpegru1"] );
		this.dlgMain.SetDropDown( this.dlgMain.ddResizeJPEGW2, this.params["jpegwu2"] );
		this.dlgMain.SetDropDown( this.dlgMain.ddResizeJPEGH2, this.params["jpeghu2"] );
		this.dlgMain.SetDropDown( this.dlgMain.ddResizeJPEGR2, this.params["jpegru2"] );
		this.dlgMain.SetDropDown( this.dlgMain.ddResizeJPEGW3, this.params["jpegwu3"] );
		this.dlgMain.SetDropDown( this.dlgMain.ddResizeJPEGH3, this.params["jpeghu3"] );
		this.dlgMain.SetDropDown( this.dlgMain.ddResizeJPEGR3, this.params["jpegru3"] );

		this.dlgMain.lastJPEG1Units = this.params["jpegwu1"];
		this.dlgMain.lastJPEG2Units = this.params["jpegwu2"];
		this.dlgMain.lastJPEG3Units = this.params["jpegwu3"];

	}

	// transfer from the dialog widgets to my internal params
	this.GetParamsFromDialog = function() {
		
		var p = this.params;
		var d = this.dlgMain;
		
		if ( ! this.runningFromBridge ) {
			p["useopen"] = d.rbUseOpen.value;
			p["source"] = d.sourceLongText;
		}

		p["saveinsame"] = d.rbSaveInSame.value;
		p["dest"] = d.destLongText;
		p["makefolders"] = d.cbMakeFolders.value;
		p["folder1"] = d.etFolder1.text;
		p["folder2"] = d.etFolder2.text;
		p["folder3"] = d.etFolder3.text;
		p["jpeg1"] = d.cbJPEG1.value;
		p["jpeg2"] = d.cbJPEG2.value;
		p["jpeg3"] = d.cbJPEG3.value;
		p["converticc1"] = d.cbConvertICC1.value;
		p["converticc2"] = d.cbConvertICC2.value;
		p["converticc3"] = d.cbConvertICC3.value;
		p["q1"] = d.etQuality1.text;
		p["q2"] = d.etQuality2.text;
		p["q3"] = d.etQuality3.text;
		p["runaction1"] = d.cbRunAction1.value;
		p["runaction2"] = d.cbRunAction2.value;
		p["runaction3"] = d.cbRunAction3.value;
		p["jpegresize1"] = d.cbResizeJPEG1.value;
		p["jpegh1"] = d.etResizeJPEGH1.text;
		p["jpegw1"] = d.etResizeJPEGW1.text;
		p["jpegr1"] = d.etResizeJPEGR1.text;
		p["jpeghu1"] = d.ddResizeJPEGH1.selection.text;
		p["jpegwu1"] = d.ddResizeJPEGW1.selection.text;
		p["jpegru1"] = d.ddResizeJPEGR1.selection.text;
		p["jpegresize2"] = d.cbResizeJPEG2.value;
		p["jpegh2"] = d.etResizeJPEGH2.text;
		p["jpegw2"] = d.etResizeJPEGW2.text;
		p["jpegr2"] = d.etResizeJPEGR2.text;
		p["jpeghu2"] = d.ddResizeJPEGH2.selection.text;
		p["jpegwu2"] = d.ddResizeJPEGW2.selection.text;
		p["jpegru2"] = d.ddResizeJPEGR2.selection.text;
		p["jpegresize3"] = d.cbResizeJPEG3.value;
		p["jpegh3"] = d.etResizeJPEGH3.text;
		p["jpegw3"] = d.etResizeJPEGW3.text;
		p["jpegr3"] = d.etResizeJPEGR3.text;
		p["jpeghu3"] = d.ddResizeJPEGH3.selection.text;
		p["jpegwu3"] = d.ddResizeJPEGW3.selection.text;
		p["jpegru3"] = d.ddResizeJPEGR3.selection.text;
		p["actionset"] = d.ddSet.selection.text;
		p["action"] = d.ddAction.selection.text;
		p["info"] = d.etCopyrightInfo.text;
		p["icc"] = d.cbIncludeICC.value;
	}
	
	// routine for running the dialog and it's interactions
	this.RunDialog = function () {

		this.dlgMain.btnCancel.onClick = function() { 
			var d = FindDialog( this );
			d.close( false );
		}

		if ( ! this.runningFromBridge ) {
			this.dlgMain.rbUseOpen.onClick = function() {
				var d = FindDialog( this );
				d.rbUseFolder.value = ! this.value;
				d.btnSource.enabled = ! this.value;
				d.stSource.enabled = ! this.value;
			}

			this.dlgMain.rbUseFolder.onClick = function() {
				var d = FindDialog( this );
				d.rbUseOpen.value = ! this.value;
				d.btnSource.enabled = this.value;
				d.stSource.enabled = this.value;
			}

			this.dlgMain.btnSource.onClick = function() { 
				var d = FindDialog( this );
				var selFolder = Folder.selectDialog( strPickSource, this.parent.parent.sourceLongText );
				if ( selFolder != null ) {
					d.sourceLongText = selFolder.fsName.toString()
					d.stSource.text = CreateShortFileName( d.sourceLongText );
				}
				d.defaultElement.active = true;
			}

		}

		this.dlgMain.rbSaveInSame.onClick = function() {
			var d = FindDialog( this );
			d.rbSaveInNew.value = ! this.value;
			d.btnDest.enabled = ! this.value;
			d.stDest.enabled = ! this.value;
		}
		
		this.dlgMain.rbSaveInNew.onClick = function() {
			var d = FindDialog( this );
			d.rbSaveInSame.value = ! this.value;
			d.btnDest.enabled = this.value;
			d.stDest.enabled = this.value;
		}

		this.dlgMain.SetDropDown = function ( dd, strSet ) {
			var index = 0;
			for ( var i = 0; i < dd.items.length; i++ ) {
				if ( dd.items[ i ].toString() == strSet ) {
					index = i;
					break;
				}
			}
			if ( dd.items.length > 0 ) {
				dd.items[ index ].selected = true;
			}
			return index;
		}

		this.dlgMain.btnLoad.onClick = function() { 
			var d = FindDialog( this );
			var loadFile = null;
			Folder.current = Folder( "~/" );
			if ( File.fs == "Windows" ) {
				loadFile = File.openDialog( strPickXML, strPickXMLWin );
			} else {
				loadFile = File.openDialog( strPickXML, MacXMLFilter );
			}
			if ( loadFile != null ) {
				d.ip.LoadParamsFromDisk( loadFile );
				d.ip.InitDialog();
				d.loadFromDisk = true;
				d.ip.ForceDialogUpdate();
			}
			d.defaultElement.active = true;
		}
		
		this.dlgMain.btnSave.onClick = function() { 
			var d = FindDialog( this );
			var saveFile = null;
			Folder.current = Folder( "~/" );
			if ( File.fs == "Windows" ) {
				saveFile = File.saveDialog( strPickXMLSave, strPickXMLWin );
			} else {
				saveFile = File.saveDialog( strPickXMLSave, MacXMLFilter );
			}
			if ( saveFile != null ) {
				d.ip.GetParamsFromDialog();
				d.ip.SaveParamsToDisk( saveFile );
			}
			d.defaultElement.active = true;
		}

		this.dlgMain.btnDest.onClick = function() { 
			var d = FindDialog( this );
			var selFolder = Folder.selectDialog( strPickDest, this.parent.parent.destLongText );
			if ( selFolder != null ) {
				d.destLongText = selFolder.fsName.toString()
				d.stDest.text = CreateShortFileName( d.destLongText );
			}
			d.defaultElement.active = true;
		}
		
		this.dlgMain.cbMakeFolders.onClick = function() {
		    var d = FindDialog( this );
		    d.icn1.enabled = this.value;
		    d.icn2.enabled = this.value;
		    d.icn3.enabled = this.value;
		    d.etFolder1.enabled = this.value;
		    d.etFolder2.enabled = this.value;
		    d.etFolder3.enabled = this.value;
		}
		
		this.dlgMain.cbJPEG1.onClick = function() {
			var d = FindDialog( this );
			d.stQuality1.enabled = this.value;
			d.etQuality1.enabled = this.value;
			d.cbConvertICC1.enabled = this.value;
			d.cbRunAction1.enabled = this.value;
			d.cbResizeJPEG1.enabled = this.value;
			d.cbResizeJPEG1.onClick();
		}

		this.dlgMain.cbJPEG2.onClick = function() {
			var d = FindDialog( this );
			d.stQuality2.enabled = this.value;
			d.etQuality2.enabled = this.value;
			d.cbConvertICC2.enabled = this.value;
			d.cbRunAction2.enabled = this.value;
			d.cbResizeJPEG2.enabled = this.value;
			d.cbResizeJPEG2.onClick();
		}
		
		this.dlgMain.cbJPEG3.onClick = function() {
			var d = FindDialog( this );
			d.stQuality3.enabled = this.value;
			d.etQuality3.enabled = this.value;
			d.cbConvertICC3.enabled = this.value;
			d.cbRunAction3.enabled = this.value;
			d.cbResizeJPEG3.enabled = this.value;
			d.cbResizeJPEG3.onClick();
		}
		
		this.dlgMain.cbResizeJPEG1.onClick = function( inValue ) {
			var d = FindDialog( this );
			var realValue = this.value;
			if ( ! d.cbJPEG1.value )
				realValue = false;
			d.stResizeJPEGW1.enabled = realValue;
			d.etResizeJPEGW1.enabled = realValue;
			d.ddResizeJPEGW1.enabled = realValue;
			d.stResizeJPEGH1.enabled = realValue;
			d.etResizeJPEGH1.enabled = realValue;
			d.ddResizeJPEGH1.enabled = realValue;
			d.stResizeJPEGR1.enabled = realValue;
			d.etResizeJPEGR1.enabled = realValue;
			d.ddResizeJPEGR1.enabled = realValue;
		}
		
		this.dlgMain.cbResizeJPEG2.onClick = function( inValue ) {
			var d = FindDialog( this );
			var realValue = this.value;
			if ( ! d.cbJPEG2.value )
				realValue = false;
			d.stResizeJPEGW2.enabled = realValue;
			d.etResizeJPEGW2.enabled = realValue;
			d.ddResizeJPEGW2.enabled = realValue;
			d.stResizeJPEGH2.enabled = realValue;
			d.etResizeJPEGH2.enabled = realValue;
			d.ddResizeJPEGH2.enabled = realValue;
			d.stResizeJPEGR2.enabled = realValue;
			d.etResizeJPEGR2.enabled = realValue;
			d.ddResizeJPEGR2.enabled = realValue;
		}
		
		this.dlgMain.cbResizeJPEG3.onClick = function( inValue ) {
			var d = FindDialog( this );
			var realValue = this.value;
			if ( ! d.cbJPEG3.value )
				realValue = false;
			d.stResizeJPEGW3.enabled = realValue;
			d.etResizeJPEGW3.enabled = realValue;
			d.ddResizeJPEGW3.enabled = realValue;
			d.stResizeJPEGH3.enabled = realValue;
			d.etResizeJPEGH3.enabled = realValue;
			d.ddResizeJPEGH3.enabled = realValue;
			d.stResizeJPEGR3.enabled = realValue;
			d.etResizeJPEGR3.enabled = realValue;
			d.ddResizeJPEGR3.enabled = realValue;
		}
		
		this.dlgMain.ddResizeJPEGW1.onChange = function() {
		    var d = FindDialog( this );
		    if ( d.lastJPEG1Units != this.selection.text ) {
		        if ( ( d.lastJPEG1Units != 'percent' ) && ( this.selection.text != 'percent' ) ) {
			        var nu = UnitsToUnits( d.etResizeJPEGW1.text, d.lastJPEG1Units, this.selection.text, ResolutionInPixelsPerInch( d.etResizeJPEGR1.text, d.ddResizeJPEGR1.selection.text ), undefined );
			        d.etResizeJPEGW1.text = nu.toString();
			        nu = UnitsToUnits( d.etResizeJPEGH1.text, d.lastJPEG1Units, this.selection.text, ResolutionInPixelsPerInch( d.etResizeJPEGR1.text, d.ddResizeJPEGR1.selection.text ), undefined );
		 	        d.etResizeJPEGH1.text = nu.toString();
		 	    } 
		        d.lastJPEG1Units = this.selection.text;
		    }
		    d.SetDropDown( d.ddResizeJPEGH1, this.selection.text );
		}

		this.dlgMain.ddResizeJPEGH1.onChange = function() {
		    var d = FindDialog( this );
		    if ( d.lastJPEG1Units != this.selection.text ) {
		        if ( ( d.lastJPEG1Units != 'percent' ) && ( this.selection.text != 'percent' ) ) {
    		        var nu = UnitsToUnits( d.etResizeJPEGW1.text, d.lastJPEG1Units, this.selection.text, ResolutionInPixelsPerInch( d.etResizeJPEGR1.text, d.ddResizeJPEGR1.selection.text ), undefined );
	    	        d.etResizeJPEGW1.text = nu.toString();
		            nu = UnitsToUnits( d.etResizeJPEGH1.text, d.lastJPEG1Units, this.selection.text, ResolutionInPixelsPerInch( d.etResizeJPEGR1.text, d.ddResizeJPEGR1.selection.text ), undefined );
		            d.etResizeJPEGH1.text = nu.toString();
		        }
		        d.lastJPEG1Units = this.selection.text;
		    }
		    d.SetDropDown( d.ddResizeJPEGW1, this.selection.text );
		}

		this.dlgMain.ddResizeJPEGR1.onChange = function() {
           var d = FindDialog( this );
		    if ( d.initisover1 ) {
		        var n = Number( d.etResizeJPEGR1.text );
		        if ( this.selection.text.search( "inch" ) != -1 ) {
    		        n = n * 2.54;
		        } else {
    		        n = n / 2.54;
		        }
		        d.etResizeJPEGR1.text = n.toString();
		    } else {
		    	d.initisover1 = true;
		    }
		}

		this.dlgMain.ddResizeJPEGW2.onChange = function() {
		    var d = FindDialog( this );
		    if ( d.lastJPEG2Units != this.selection.text ) {
		        if ( ( d.lastJPEG2Units != 'percent' ) && ( this.selection.text != 'percent' ) ) {
    		        var nu = UnitsToUnits( d.etResizeJPEGW2.text, d.lastJPEG2Units, this.selection.text, ResolutionInPixelsPerInch( d.etResizeJPEGR2.text, d.ddResizeJPEGR2.selection.text ), undefined );
	    	        d.etResizeJPEGW2.text = nu.toString();
		            nu = UnitsToUnits( d.etResizeJPEGH2.text, d.lastJPEG2Units, this.selection.text, ResolutionInPixelsPerInch( d.etResizeJPEGR2.text, d.ddResizeJPEGR2.selection.text ), undefined );
		            d.etResizeJPEGH2.text = nu.toString();
		        }
		        d.lastJPEG2Units = this.selection.text;
		    }
		    d.SetDropDown( d.ddResizeJPEGH2, this.selection.text );
		}

		this.dlgMain.ddResizeJPEGH2.onChange = function() {
		    var d = FindDialog( this );
		    if ( d.lastJPEG2Units != this.selection.text ) {
		        if ( ( d.lastJPEG2Units != 'percent' ) && ( this.selection.text != 'percent' ) ) {
    		        var nu = UnitsToUnits( d.etResizeJPEGW2.text, d.lastJPEG2Units, this.selection.text, ResolutionInPixelsPerInch( d.etResizeJPEGR2.text, d.ddResizeJPEGR2.selection.text ), undefined );
	    	        d.etResizeJPEGW2.text = nu.toString();
		            nu = UnitsToUnits( d.etResizeJPEGH2.text, d.lastJPEG2Units, this.selection.text, ResolutionInPixelsPerInch( d.etResizeJPEGR2.text, d.ddResizeJPEGR2.selection.text ), undefined );
		            d.etResizeJPEGH2.text = nu.toString();
		        }
		        d.lastJPEG2Units = this.selection.text;
		    }
		    d.SetDropDown( d.ddResizeJPEGW2, this.selection.text );
		}

		this.dlgMain.ddResizeJPEGR2.onChange = function() {
           var d = FindDialog( this );
		    if ( d.initisover2 ) {
		        var d = FindDialog( this );
		        var n = Number( d.etResizeJPEGR2.text );
		        if ( this.selection.text.search( "inch" ) != -1 ) {
		            n = n * 2.54;
		        } else {
		            n = n / 2.54;
		        }
		        d.etResizeJPEGR2.text = n.toString();
		    } else {
		    	d.initisover2 = true;
		    }
		}

		this.dlgMain.ddResizeJPEGW3.onChange = function() {
		    var d = FindDialog( this );
		    if ( d.lastJPEG3Units != this.selection.text ) {
		        if ( ( d.lastJPEG3Units != 'percent' ) && ( this.selection.text != 'percent' ) ) {
    		        var nu = UnitsToUnits( d.etResizeJPEGW3.text, d.lastJPEG3Units, this.selection.text, ResolutionInPixelsPerInch( d.etResizeJPEGR3.text, d.ddResizeJPEGR3.selection.text ), undefined );
	    	        d.etResizeJPEGW3.text = nu.toString();
		            nu = UnitsToUnits( d.etResizeJPEGH3.text, d.lastJPEG3Units, this.selection.text, ResolutionInPixelsPerInch( d.etResizeJPEGR3.text, d.ddResizeJPEGR3.selection.text ), undefined );
		            d.etResizeJPEGH3.text = nu.toString();
		        }
		        d.lastJPEG3Units = this.selection.text;
		    }
		    d.SetDropDown( d.ddResizeJPEGH3, this.selection.text );
		}

		this.dlgMain.ddResizeJPEGH3.onChange = function() {
		    var d = FindDialog( this );
		    if ( d.lastJPEG3Units != this.selection.text ) {
		        if ( ( d.lastJPEG3Units != 'percent' ) && ( this.selection.text != 'percent' ) ) {
    		        var nu = UnitsToUnits( d.etResizeJPEGW3.text, d.lastJPEG3Units, this.selection.text, ResolutionInPixelsPerInch( d.etResizeJPEGR3.text, d.ddResizeJPEGR3.selection.text ), undefined );
	    	        d.etResizeJPEGW3.text = nu.toString();
		            nu = UnitsToUnits( d.etResizeJPEGH3.text, d.lastJPEG3Units, this.selection.text, ResolutionInPixelsPerInch( d.etResizeJPEGR3.text, d.ddResizeJPEGR3.selection.text ), undefined );
		            d.etResizeJPEGH3.text = nu.toString();
		        }
		        d.lastJPEG3Units = this.selection.text;
		    }
		    d.SetDropDown( d.ddResizeJPEGW3, this.selection.text );
		}

		this.dlgMain.ddResizeJPEGR3.onChange = function() {
           var d = FindDialog( this );
		    if ( d.initisover3 ) {
		        var d = FindDialog( this );
		        var n = Number( d.etResizeJPEGR3.text );
		        if ( this.selection.text.search( "inch" ) != -1 ) {
		            n = n * 2.54;
		        } else {
		            n = n / 2.54;
		        }
		        d.etResizeJPEGR3.text = n.toString();
		    } else {
		    	d.initisover3 = true;
		    }
		}

		if ( this.actionInfo.length > 0 ) {
			for ( var i = 0; i < this.actionInfo.length; i++ ) {
				this.dlgMain.ddSet.add( "item", this.actionInfo[i].name );
			}
			this.dlgMain.ddSet.items[0].selected = true;
			this.dlgMain.ddSet.onChange = function( forcedSelectionIndex ) {
				var index = 0;
				var d = FindDialog( this );
				d.ddAction.removeAll();
				if ( undefined == forcedSelectionIndex ) {
					forcedSelectionIndex = this.selection.index;
					if ( undefined != d.ip.actionIndex ) {
						index = d.ip.actionIndex;
						d.ip.actionIndex = undefined;
					}
				}
				for ( var i = 0; i < d.ip.actionInfo[ forcedSelectionIndex ].children.length; i++ ) {
					d.ddAction.add( "item", d.ip.actionInfo[ forcedSelectionIndex ].children[ i ].name );
				}
				if ( d.ddAction.items.length > 0 ) {
					d.ddAction.items[ index ].selected = true;
				}
				if ( null != this.selection ) {
					this.helpTip = this.selection.text;
				}
			}
		} else {
			this.dlgMain.grpLeft.grpAction.enabled = false;
		}
	
		this.dlgMain.ddAction.onChange = function() {
			this.helpTip = this.selection.text;
		}

		this.dlgMain.btnRun.onClick = function () {
			var testFolder = null;
			var d = FindDialog( this );
			if ( ! this.runningFromBridge ) {
				if ( this.rbUseFolder ) {
					if ( d.sourceLongText.length > 0 && d.sourceLongText[0] != '.' ) {
						testFolder = new Folder( d.sourceLongText );
						if ( !testFolder.exists ) {
							alert( strSpecifySource );
							return;
						}
					} else {
						alert( strSpecifySource );
						return;
					}
				}
			}

			if ( d.rbSaveInNew.value ) {
				if ( d.destLongText.length > 0 && d.destLongText[0] != '.' ) {
					testFolder = new Folder( d.destLongText );
					if ( !testFolder.exists ) {
						alert( strSpecifyDest );
						return;
					}
				} else {
					alert( strSpecifyDest );
					return;
				}
			}
						
			if ( d.cbJPEG1.value ) {
				var q = Number( d.etQuality1.text );
				if ( q < 0 || q > 12 || isNaN( q ) ) {
					alert( strJPEGQuality );
					return;
				}

    			if ( d.cbResizeJPEG1.value ) {
    			    var whcount = 2;
	    			var w = Number( d.etResizeJPEGW1.text );
		    		var h = Number( d.etResizeJPEGH1.text );
			        if ( w == 0 || isNaN( w ) ) {
			            whcount--;
			        }
			        if ( h == 0 || isNaN( h ) ) {
			            whcount--;
			        }
			        if ( whcount == 0 ) {
					    alert( strJPEGWandH );
					    return;
				    }
			    }
			}

			if ( d.cbJPEG2.value ) {
				var q = Number( d.etQuality2.text );
				if ( q < 0 || q > 12 || isNaN( q ) ) {
					alert( strJPEGQuality );
					return;
				}

			    if ( d.cbResizeJPEG2.value ) {
    			    var whcount = 2;
	    			var w = Number( d.etResizeJPEGW2.text );
		    		var h = Number( d.etResizeJPEGH2.text );
			        if ( w == 0 || isNaN( w ) ) {
			            whcount--;
			        }
			        if ( h == 0 || isNaN( h ) ) {
			            whcount--;
			        }
			        if ( whcount == 0 ) {
					    alert( strJPEGWandH );
					    return;
				    }
			    }
			}

			if ( d.cbJPEG3.value ) {
				var q = Number( d.etQuality3.text );
				if ( q < 0 || q > 12 || isNaN( q ) ) {
					alert( strJPEGQuality );
					return;
				}

			    if ( d.cbResizeJPEG3.value ) {
    			    var whcount = 2;
	    			var w = Number( d.etResizeJPEGW3.text );
		    		var h = Number( d.etResizeJPEGH3.text );
			        if ( w == 0 || isNaN( w ) ) {
			            whcount--;
			        }
			        if ( h == 0 || isNaN( h ) ) {
			            whcount--;
			        }
			        if ( whcount == 0 ) {
					    alert( strJPEGWandH );
					    return;
				    }
			    }
			}



			// make sure they have at least one file format specified for output
			var outputCount = 0;
			if ( d.cbJPEG1.value ) {
				outputCount++;
			}
			if ( d.cbJPEG2.value ) {
				outputCount++;
			}
			if ( d.cbJPEG3.value ) {
				outputCount++;
			}

			if ( outputCount == 0 ) {
				alert( strOneType );
				return;
			}

			d.ip.GetParamsFromDialog()
			
			d.close( true );
		}

		this.InitDialog();

		this.ForceDialogUpdate();
		
		this.dlgMain.center();
		
		// this.dlgMain.initisover = true;
		
		return this.dlgMain.show();

	}

	// if I get here then the dialog params are ok and it is time
	// to do what they want
	this.Execute = function()  {
		var cameraRawParams = new Object();
		cameraRawParams.desc = undefined;
		cameraRawParams.useDescriptor = false;
		cameraRawParams.fileName = this.params["source"] + "/x";
		if ( ( this.runningFromBridge || ! this.params["useopen"] ) && this.params["open"] ) {
			cameraRawParams.useDescriptor = true;
			cameraRawParams.fileName = this.GetFirstFile( this.params["source"] );
			if ( ! IsFileOneOfThese( cameraRawParams.fileName, gFilesToSkip ) ) {
				if ( this.OpenCameraRaw( cameraRawParams, true ) ) {
					this.AdjustFile( cameraRawParams );
					if ( this.params["saveinsame"] ) {
						var destFolder = File( cameraRawParams.fileName ).parent.toString();
					} else {
						var destFolder = this.params["dest"];
					}
					this.SaveFile( cameraRawParams.fileName, destFolder );
					app.activeDocument.close( SaveOptions.DONOTSAVECHANGES );
				} else {
					return;
				}
			} else {
				alert( strNotaRAW + cameraRawParams.fileName );
				return;
			}
		}
		this.OpenAdjustSaveTheRest( cameraRawParams );
	}
	
	// the heart of the script is this routine and Execute
	// loop through all the files and save accordingly
	this.OpenAdjustSaveTheRest = function( inCameraRawParams ) {
		var firstFileAsLowerCaseString = "";
		if ( inCameraRawParams.fileName != undefined ) {
			firstFileAsLowerCaseString = inCameraRawParams.fileName.toString().toLowerCase();
		}
		var inputFiles;
		if ( this.runningFromBridge ) {
			inputFiles = this.filesFromBridge;
		} else if ( this.params["useopen"] ) {
			inputFiles = OpenDocs();
		} else {
			inputFiles = Folder ( this.params["source"] ).getFiles ();
		}

		for (var i = 0; i < inputFiles.length; i++) {

			var thisFileAsLowerCaseString = inputFiles[i].toString().toLowerCase();

			if ( ( inputFiles[i] instanceof File && 
			       ! inputFiles[i].hidden && 
			       firstFileAsLowerCaseString != thisFileAsLowerCaseString ) || 
			      this.params["useopen"] ) {

				inCameraRawParams.fileName = inputFiles[i].fullName.toString();

				if ( ! IsFileOneOfThese( inCameraRawParams.fileName, gFilesToSkip ) ) {
					if ( ! this.runningFromBridge && this.params["useopen"] ) {
						app.activeDocument = inputFiles[ i ];
						app.activeDocument.duplicate();
					} else {
						this.OpenCameraRaw( inCameraRawParams, false, DialogModes.NO );
					}

					this.AdjustFile( inCameraRawParams );

					if ( this.params["saveinsame"] ) {
						var destFolder = inputFiles[i].parent.toString();
						if ( ! this.runningFromBridge && this.params["useopen"] ) {
							var destFolder = inputFiles[i].fullName.parent.toString();
							inCameraRawParams.fileName = app.activeDocument.name;
						}
					} else {
						var destFolder = this.params["dest"];
					}

					this.SaveFile( inCameraRawParams.fileName, destFolder  );
					app.activeDocument.close( SaveOptions.DONOTSAVECHANGES );
				}
			}
		}
	}
	
	// using the dialog adjust the active document
	this.AdjustFile = function ( inOutCameraRawParams ) {
		var docRef = app.activeDocument;

		if ( this.params["info"] != "" ) {
			docRef.info.copyrightNotice = this.params["info"];
			docRef.info.copyrighted = CopyrightedType.COPYRIGHTEDWORK;
		}
	}
	
	// I can save in three formats, JPEG, JPEG, and JPEG
	// you can save it in any file type you like as long as it is JPEG ( Henry Ford reference here. )
	this.SaveFile = function ( inFileName, inFolderLocation ) {
		var lastDot = inFileName.lastIndexOf( "." );
		if ( lastDot == -1 ) {
			lastDot = inFileName.length;
		}
		var fileNameNoPath = inFileName.substr( 0, lastDot );
		var lastSlash = fileNameNoPath.lastIndexOf( "/" );
		fileNameNoPath = fileNameNoPath.substr( lastSlash + 1, fileNameNoPath.length );
		
		// these are all the same, I didn't make a routine because the param list is huge
		// any modifications in the if need to be copy pasted
		if ( this.params["jpeg1"] ) {
		    var subFolderText = inFolderLocation + "/";
		    if ( this.params["makefolders"] ) {
		        subFolderText = subFolderText + this.params["folder1"] + "/";
		    }
			Folder( subFolderText ).create();
			var historyState = app.activeDocument.activeHistoryState;
			app.activeDocument.flatten();
			app.activeDocument.bitsPerChannel = BitsPerChannelType.EIGHT;
			RemoveAlphaChannels();
			if ( DocumentMode.RGB != app.activeDocument.mode ) {
				ConditionalModeChangeToRGB();
			}
			if ( this.params["converticc1"] ) {
				ConvertTosRGBProfile();
			}
			if ( this.params["jpegresize1"] ) {
			    var r = ResolutionInPixelsPerInch( this.params["jpegr1"], this.params["jpegru1"] )
			    if ( r == 0 || isNaN( r ) ) {
			        r = app.activeDocument.resolution;
			    }
			    if ( app.activeDocument.resolution != r ) {
			        app.activeDocument.resizeImage( undefined, undefined, r, ResampleMethod.NONE );
			    }
			    var w = Number( this.params["jpegw1"] );
			    var h = Number( this.params["jpegh1"] );
			    if ( w == 0 || isNaN( w ) ) {
			        w = h;
			    }
			    if ( h == 0 || isNaN( h ) ) {
			        h = w;
			    }    
                w = UnitsToPixels( w, this.params["jpegwu1"], r, app.activeDocument.width.as( 'px' ) );
                h = UnitsToPixels( h, this.params["jpeghu1"], r, app.activeDocument.height.as( 'px' ) );
				FitImage( w, h ); 
			}
			if ( this.params["runaction1"] ) {
				app.doAction( this.params["action"], this.params["actionset"] );
			}
			var uniqueFileName = CreateUniqueFileName( subFolderText, fileNameNoPath, ".jpg" );
			SaveAsJPEG( uniqueFileName, this.params["q1"], this.params["icc"] );
			app.activeDocument.activeHistoryState = historyState;
		}
		
		if ( this.params["jpeg2"] ) {
		    var subFolderText = inFolderLocation + "/";
		    if ( this.params["makefolders"] ) {
		        subFolderText = subFolderText + this.params["folder2"] + "/";
		    }
			Folder( subFolderText ).create();
			var historyState = app.activeDocument.activeHistoryState;
			app.activeDocument.flatten();
			app.activeDocument.bitsPerChannel = BitsPerChannelType.EIGHT;
			RemoveAlphaChannels();
			if ( DocumentMode.RGB != app.activeDocument.mode ) {
				ConditionalModeChangeToRGB();
			}
			if ( this.params["converticc2"] ) {
				ConvertTosRGBProfile();
			}
			if ( this.params["jpegresize2"] ) {
			    var r = ResolutionInPixelsPerInch( this.params["jpegr2"], this.params["jpegru2"] )
			    if ( r == 0 || isNaN( r ) ) {
			        r = app.activeDocument.resolution;
			    }
			    if ( app.activeDocument.resolution != r ) {
			        app.activeDocument.resizeImage( undefined, undefined, r, ResampleMethod.NONE );
			    }
			    var w = Number( this.params["jpegw2"] );
			    var h = Number( this.params["jpegh2"] );
			    if ( w == 0 || isNaN( w ) ) {
			        w = h;
			    }
			    if ( h == 0 || isNaN( h ) ) {
			        h = w;
			    }    
                w = UnitsToPixels( w, this.params["jpegwu2"], r, app.activeDocument.width.as( 'px' ) );
                h = UnitsToPixels( h, this.params["jpeghu2"], r, app.activeDocument.height.as( 'px' ) );
				FitImage( w, h ); 
			}
			if ( this.params["runaction2"] ) {
				app.doAction( this.params["action"], this.params["actionset"] );
			}
			var uniqueFileName = CreateUniqueFileName( subFolderText, fileNameNoPath, ".jpg" );
			SaveAsJPEG( uniqueFileName, this.params["q2"], this.params["icc"] );
			app.activeDocument.activeHistoryState = historyState;
		}
		
		if ( this.params["jpeg3"] ) {
		    var subFolderText = inFolderLocation + "/";
		    if ( this.params["makefolders"] ) {
		        subFolderText = subFolderText + this.params["folder3"] + "/";
		    }
			Folder( subFolderText ).create();
			var historyState = app.activeDocument.activeHistoryState;
			app.activeDocument.flatten();
			app.activeDocument.bitsPerChannel = BitsPerChannelType.EIGHT;
			RemoveAlphaChannels();
			if ( DocumentMode.RGB != app.activeDocument.mode ) {
				ConditionalModeChangeToRGB();
			}
			if ( this.params["converticc3"] ) {
				ConvertTosRGBProfile();
			}
			if ( this.params["jpegresize3"] ) {
			    var r = ResolutionInPixelsPerInch( this.params["jpegr3"], this.params["jpegru3"] )
			    if ( r == 0 || isNaN( r ) ) {
			        r = app.activeDocument.resolution;
			    }
			    if ( app.activeDocument.resolution != r ) {
			        app.activeDocument.resizeImage( undefined, undefined, r, ResampleMethod.NONE );
			    }
			    var w = Number( this.params["jpegw3"] );
			    var h = Number( this.params["jpegh3"] );
			    if ( w == 0 || isNaN( w ) ) {
			        w = h;
			    }
			    if ( h == 0 || isNaN( h ) ) {
			        h = w;
			    }    
                w = UnitsToPixels( w, this.params["jpegwu3"], r, app.activeDocument.width.as( 'px' ) );
                h = UnitsToPixels( h, this.params["jpeghu3"], r, app.activeDocument.height.as( 'px' ) );
				FitImage( w, h ); 
			}
			if ( this.params["runaction3"] ) {
				app.doAction( this.params["action"], this.params["actionset"] );
			}
			var uniqueFileName = CreateUniqueFileName( subFolderText, fileNameNoPath, ".jpg" );
			SaveAsJPEG( uniqueFileName, this.params["q3"], this.params["icc"] );
			app.activeDocument.activeHistoryState = historyState;
		}
	}
		
	// open a camera raw file returning the camera raw action desc
	this.OpenCameraRaw = function( inOutCameraRawParams, updateCRDesc, inDialogMode ) {
		var keyNull = charIDToTypeID( 'null' );
		var keyAs = charIDToTypeID( 'As  ' );
		var adobeCameraRawID = stringIDToTypeID( "Adobe Camera Raw" );
		var desc = new ActionDescriptor();
		desc.putPath( keyNull, File( inOutCameraRawParams.fileName ) );
		if ( inOutCameraRawParams.desc != undefined && inOutCameraRawParams.useDescriptor &&
		     IsFileOneOfThese( inOutCameraRawParams.fileName, gFilesForCameraRaw ) ) {
			desc.putObject( keyAs, adobeCameraRawID, inOutCameraRawParams.desc );
		}
		if ( inDialogMode == undefined ) {
			inDialogMode = DialogModes.ALL;
		}
		var returnDesc = executeAction( charIDToTypeID( 'Opn ' ), desc, inDialogMode );
		if ( returnDesc.hasKey( keyAs ) ) {
			if (updateCRDesc)
				inOutCameraRawParams.desc = returnDesc.getObjectValue( keyAs, adobeCameraRawID );
			if ( returnDesc.hasKey( keyNull ) ) {
				inOutCameraRawParams.fileName = returnDesc.getPath( keyNull ).toString();
				desc = null;
				$.gc();
				return true;
			}
		}
		desc = null;
		$.gc();
		return false;
	}
	
	// given a folder return the first valid file in that folder
	this.GetFirstFile = function ( inFolder ) {
		if ( inFolder.length > 0 && inFolder[0] != '.' ) {
			var fileList = Folder( inFolder ).getFiles();
			for ( var i = 0; i < fileList.length; i++) {
				if ( fileList[i] instanceof File && ! fileList[i].hidden && ! IsFileOneOfThese( fileList[i], gFilesToSkip ) ) {
					return fileList[i].toString();
				}
			}
		}
		return "";
	}
	
	///////////////////////////////////////////////////////////////////////////
	// Function: ConfigForBridge
	// Usage: see if the Bridge app is running this script
	// Input: gFilesFromBridge is a variable that is defined in photoshop.jsx
	// Return: runningFromBridge and filesFromBridge are initialized
	///////////////////////////////////////////////////////////////////////////
	this.ConfigForBridge = function() {
		try {
			this.filesFromBridge = gFilesFromBridge;
			if ( this.filesFromBridge.length > 0 ) {
				this.runningFromBridge = true;
			} else {
				this.runningFromBridge = false;
			}
		}
		catch( e ) { 
			this.runningFromBridge = false;
			this.filesFromBridge = undefined;
		}
	}


	// initialize properties

	this.params = this.InitParams();

	this.actionInfo = GetActionSetInfo();
	
	this.ConfigForBridge();
	
}


function SaveAsJPEG( inFileName, inQuality, inEmbedICC ) {
	var jpegOptions = new JPEGSaveOptions();
	jpegOptions.quality = inQuality;
	jpegOptions.embedColorProfile = inEmbedICC;
	app.activeDocument.saveAs( File( inFileName ), jpegOptions );
}
	
// use the fit image automation plug-in to do this work for me
function FitImage( inWidth, inHeight ) {
	if ( inWidth == undefined || inHeight == undefined ) {
		alert( strWidthAndHeight );
		return;
	}
	var desc = new ActionDescriptor();
	var unitPixels = charIDToTypeID( '#Pxl' );
	desc.putUnitDouble( charIDToTypeID( 'Wdth' ), unitPixels, inWidth );
	desc.putUnitDouble( charIDToTypeID( 'Hght' ), unitPixels, inHeight );
	var runtimeEventID = stringIDToTypeID( "3caa3434-cb67-11d1-bc43-0060b0a13dc4" );	
	executeAction( runtimeEventID, desc, DialogModes.NO );
	desc = null;
	$.gc();
}

// a very crude xml parser, this reads the "Tag" of the <Tag>Data</Tag>
function ReadHeader( inFile ) {
	var returnValue = "";
	if ( ! inFile.eof ) {
		var c = "";
		while ( c != "<" && ! inFile.eof ) {
			c = inFile.read( 1 );
		}
		while ( c != ">" && ! inFile.eof ) {
			c = inFile.read( 1 );
			if ( c != ">" ) {
				returnValue += c;
			}
		}
	} else {
		returnValue = "end of file";
	}
	return returnValue;
}

// very crude xml parser, this reads the "Data" of the <Tag>Data</Tag>
function ReadData( inFile ) {
	var returnValue = "";
	if ( ! inFile.eof ) {
		var c = "";
		while ( c != "<" && ! inFile.eof ) {
			c = inFile.read( 1 );
			if ( c != "<" ) {
				returnValue += c;
			}
		}
		inFile.seek( -1, 1 );
	}
	return returnValue;
}

function CheckVersion() {
	var numberArray = version.split(".");
	if ( numberArray[0] < 9 ) {
		alert( strMustUse );
		throw( strMustUse );
	}
}

function CreateUniqueFileName( inFolder, inFileName, inExtension ) {
	inFileName = inFileName.replace(/[:\/\\*\?\"\<\>\|]/g, "_");  // '/\:*?"<>|' -> '_'
	var uniqueFileName = inFolder + inFileName + inExtension;
	var fileNumber = 1;
	while ( File( uniqueFileName ).exists ) {
		uniqueFileName = inFolder + inFileName + "_" + fileNumber + inExtension;
		fileNumber++;
	}
	return uniqueFileName;
}

function CreateShortFileName( inFileName ) {
	if ( inFileName.length > gShortFileNameLength ) {
		return "..." + inFileName.substr( inFileName.length - gShortFileNameLength + 3, gShortFileNameLength - 3 );
	}
	return inFileName;
}

function IsWindowsOS() {
	if ( $.os.search(/windows/i) != -1 ) {
		return true;
	} else {
		return false;
	}
}

function RemoveAlphaChannels() {
	var channels = app.activeDocument.channels;
	var channelCount = channels.length - 1;
	while ( channels[channelCount].kind != ChannelType.COMPONENT ) {
		channels[channelCount].remove();
		channelCount--;
	}
}

function ConvertTosRGBProfile() {
	app.activeDocument.convertProfile("sRGB IEC61966-2.1", Intent.RELATIVECOLORIMETRIC, true, true);
}


// you can't save certain characters in xml, strip them here
// this list is not complete
function GetScriptNameForXML () {
	var scriptNameForXML = new String( gStrTitle );
	var charsToStrip = Array( " ", "'", "." );
	for (var a = 0; a < charsToStrip.length; a++ )  {
		var nameArray = scriptNameForXML.split( charsToStrip[a] );
		scriptNameForXML = "";
		for ( var b = 0; b < nameArray.length; b++ ) {
			scriptNameForXML += nameArray[b];
		}
	}
	return scriptNameForXML;
}


/////////////////////////////////////////////////////////////////////
// Function: MacXMLFilter
// Input: f, file or folder to check
// Return: true or false, true if file or folder is to be displayed
/////////////////////////////////////////////////////////////////////
function MacXMLFilter( f )
{
	var xmlExtension = ".xml";
	var lCaseName = f.name;
	lCaseName.toLowerCase();
	if ( lCaseName.indexOf( xmlExtension ) == f.name.length - xmlExtension.length )
		return true;
	else if ( f.type == 'TEXT' )
		return true;
	else if ( f instanceof Folder )
		return true;
	else
		return false;
}

// figure out what I call my params file
function GetDefaultParamsFile() {
	return ( new File( app.preferencesFolder + "/" + gStrTitle + ".xml" ) );
}

function FindDialog( inItem ) {
	var w = inItem;
	while ( 'dialog' != w.type ) {
		if ( undefined == w.parent ) {
			w = null;
			break;
		}
		w = w.parent;
	}
	return w;
}

// Use this to retrieve the current files from the Bridge application
// I don't use it as I run differently only when called from the Bridge
// see photoshop.jsx
function GetFilesFromBridge() {
	var fileList;
	if ( BridgeTalk.isRunning( "bridge" ) ) {
		var bt = new BridgeTalk();
		bt.target = "bridge";
		bt.body = "var theFiles = photoshop.getBridgeFileListForAutomateCommand();theFiles.toSource();";
		bt.onResult = function( inBT ) { fileList = eval( inBT.body ); }
		bt.onError = function( inBT ) { fileList = new Array(); }
		bt.send();
		bt.pump();
		$.sleep( 100 );
		var timeOutAt = ( new Date() ).getTime() + 5000;
		var currentTime = ( new Date() ).getTime();
		while ( ( currentTime < timeOutAt ) && ( undefined == fileList ) ) {
			bt.pump();
			$.sleep( 100 );
			currentTime = ( new Date() ).getTime();
		}
	}
	if ( undefined == fileList ) {
		fileList = new Array();
	}
	return fileList; 
}


///////////////////////////////////////////////////////////////////////////////
// Function: GetActionSetInfo
// Usage: walk all the items in the action palette and record the action set
//        names and all the action children
// Input: <none>
// Return: the array of all the ActionData
// Note: This will throw an error during a normal execution. There is a bug
// in Photoshop that makes it impossible to get an acurate count of the number
// of action sets.
///////////////////////////////////////////////////////////////////////////////
function GetActionSetInfo() {
	var actionSetInfo = new Array();
	var setCounter = 1;
  	while ( true ) {
		var ref = new ActionReference();
		ref.putIndex( gClassActionSet, setCounter );
		var desc = undefined;
		try { desc = executeActionGet( ref ); }
		catch( e ) { break; }
		var actionData = new ActionData();
		if ( desc.hasKey( gKeyName ) ) {
			actionData.name = desc.getString( gKeyName );
		}
		var numberChildren = 0;
		if ( desc.hasKey( gKeyNumberOfChildren ) ) {
			numberChildren = desc.getInteger( gKeyNumberOfChildren );
		}
		if ( numberChildren ) {
			actionData.children = GetActionInfo( setCounter, numberChildren );
			actionSetInfo.push( actionData );
		}
		setCounter++;
	}
	ref = null;
	$.gc();
	return actionSetInfo;
}


///////////////////////////////////////////////////////////////////////////////
// Function: GetActionInfo
// Usage: used when walking through all the actions in the action set
// Input: action set index, number of actions in this action set
// Return: true or false, true if file or folder is to be displayed
///////////////////////////////////////////////////////////////////////////////
function GetActionInfo( setIndex, numChildren ) {
	var actionInfo = new Array();
	for ( var i = 1; i <= numChildren; i++ ) {
		var ref = new ActionReference();
		ref.putIndex( gClassAction, i );
		ref.putIndex( gClassActionSet, setIndex );
		var desc = undefined;
		desc = executeActionGet( ref );
		var actionData = new ActionData();
		if ( desc.hasKey( gKeyName ) ) {
			actionData.name = desc.getString( gKeyName );
		}
		var numberChildren = 0;
		if ( desc.hasKey( gKeyNumberOfChildren ) ) {
			numberChildren = desc.getInteger( gKeyNumberOfChildren );
		}
		actionInfo.push( actionData );
		ref = null;
	}
	$.gc();
	return actionInfo;
}


///////////////////////////////////////////////////////////////////////////////
// Function: ActionData
// Usage: this could be an action set or an action
// Input: <none>
// Return: a new Object of ActionData
///////////////////////////////////////////////////////////////////////////////
function ActionData() {
	this.name = "";
	this.children = undefined;
	this.toString = function () {
		var strTemp = this.name;
		if ( undefined != this.children ) {
			for ( var i = 0; i < this.children.length; i++ ) {
				strTemp += " " + this.children[i].toString();
			}
		}
		return strTemp;
	}
}

// we want the current open documents that have a path
// if I do inputFiles = documents then I have a reference
// and if I add documents it also adds them to my inputFiles
// I want to copy the current state
function OpenDocs() {
	var docs = new Array();
	var i = 0;
	var docIndex = 0;
	var alertNonSavedDocs = false;
	var nonSavedDocs = new Array();
	for ( var i = 0; i < app.documents.length; i++ ) {
		try {
			var temp = app.documents[ i ].fullName;
			docs[ docIndex ] = app.documents[ i ];
			docIndex++;
		}
		catch( e ) {
			if ( e.number == 8103 ) { // this document has not been saved
				alertNonSavedDocs = true;
				nonSavedDocs.push( app.documents[ i ].name );
			} else {
				throw e;
			}
		}
	}
	if ( alertNonSavedDocs ) {
		alert( strMustSaveOpen + "\r" + strFollowing + "\r( " + nonSavedDocs + " )" );
	}
	return docs;
}

function ConditionalModeChangeToRGB() {
	var eventModeChange = stringIDToTypeID( "8cba8cd6-cb66-11d1-bc43-0060b0a13dc4" );
	var descSource = new ActionDescriptor();
	var keySourceMode = charIDToTypeID( "SrcM" );
	var list = new ActionList();
	var keyCondition = charIDToTypeID( "Cndn" );
	var keyBitmap = charIDToTypeID( "UBtm" );
	var keyGrayscale = charIDToTypeID( "UGry" );
	var keyDuotone = charIDToTypeID( "UDtn" );
	var keyIndex = charIDToTypeID( "UInd" );
	var keyRGB = charIDToTypeID( "URGB" );
	var keyCMYK = charIDToTypeID( "UCMY" );
	var keyLab = charIDToTypeID( "ULab" );
	var keyMultichannel = charIDToTypeID( "UMlt" );
	list.putEnumerated( keyCondition, keyBitmap );
	list.putEnumerated( keyCondition, keyGrayscale );
	list.putEnumerated( keyCondition, keyDuotone );
	list.putEnumerated( keyCondition, keyIndex );
	list.putEnumerated( keyCondition, keyRGB );
	list.putEnumerated( keyCondition, keyCMYK );
	list.putEnumerated( keyCondition, keyLab );
	list.putEnumerated( keyCondition, keyMultichannel );
	descSource.putList( keySourceMode, list );
	var keyDestination = charIDToTypeID( "DstM" );
	var descDest = new ActionDescriptor();
	var keyRGB = charIDToTypeID( "RGBM" );
	descSource.putObject( keyDestination, keyRGB, descDest );
	executeAction( eventModeChange, descSource, DialogModes.NO );
	descSource = null;
	list = null;
	$.gc();
}

// Find all the items with a 'name' property and
// give access to them at the top level
// this way I do not have to do w.grp.grp.grp.grp and
// I can simply do w.cbItem
// manage the group changes all the time
function FlattenDialog( t, w ) {
    if ( w.type == 'dialog' || w.type == 'group' ) {
        for ( i in w ) {
            try {
	            if ( w[i] != undefined && w[i].type == 'dialog' || w[i].type == 'group' && i != 'parent' ) {
	                FlattenDialog( t, w[i] );
	            } else {
	                if ( w[i].properties != undefined && w[i].properties.idString != undefined ) {
	                    // turn this off when your done debugging your dialog code
	                    // if ( t[w[i].properties.idString] != undefined ) {
	                    //    alert('Already have a a ' + w[i].properties.idString + ' property.' ); 
	                    //}
	                    t[w[i].properties.idString] = w[i];
	                }
	            }
	        }
	        catch(e) {
	        	; // do nothing
	        } 
        }
    }
}

// Convert from one of these 'pixels', 'percent', 'inches', 'cm', 'mm', 'points', 'picas'
// to pixels
function UnitsToPixels( value, units, resolution, docValue ) {
    var n = Number( value );
    if ( units.search( "inches" ) != -1 ) {
        n = value * resolution;
        return n;
    }
    if ( units.search( "cm" ) != -1 ) {
        n = value / 2.54 * resolution;
        return n;
    }
    if ( units.search( "mm" ) != -1 ) {
        n = value / 25.4 * resolution;
        return n;
    }
    if ( units.search( "points" ) != -1 ) {
        n = value * resolution / 72.72;
        return n;
    }
    if ( units.search( "picas" ) != -1 ) {
        n = value * resolution / 72;
        return n;
    }
    if ( ( units.search( "percent" ) != -1 ) && ( docValue != undefined ) ) {
        n = value * docValue / 100.0;
        return n;
    }
    return n;
}

function PixelsToUnits( pixels, units, resolution, docValue ) {
    var n = Number( pixels );
    if ( units.search( "inches" ) != -1 ) {
        n = pixels / resolution;
        return n;
    }
    if ( units.search( "cm" ) != -1 ) {
        n = pixels * 2.54 / resolution;
        return n;
    }
    if ( units.search( "mm" ) != -1 ) {
        n = pixels * 25.4 / resolution;
        return n;
    }
    if ( units.search( "points" ) != -1 ) {
        n = pixels / resolution * 72.72;
        return n;
    }
    if ( units.search( "picas" ) != -1 ) {
        n = pixels / resolution * 72;
        return n;
    }
    if ( ( units.search( "percent" ) != -1 ) && ( docValue != undefined ) ) {
        n = pixels / docValue * 100.0;
        return n;
    }
    return n;
}

function UnitsToUnits( value, fromUnits, toUnits, resolution, docValue ) {
    var p = UnitsToPixels( value, fromUnits, resolution, docValue );
    p = PixelsToUnits( p, toUnits, resolution, docValue );
    return p;
}

// Pass in text for the resolution and the units (straight from the dropdowns and edit texts
// return is number in pixels per inch
function ResolutionInPixelsPerInch( resolution, units ) {
    var n = Number( resolution );
    if ( units.search( "inch" ) == -1 ) {
        n = n * 2.54;
	}
	return n;
}
// Dr. Brown's 1-2-3 JPEG.jsx
